package com.jesuscb.footcoach.model.firebase.core


import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.database.DatabaseReference
import com.google.firebase.database.FirebaseDatabase
import com.jesuscb.footcoach.model.firebase.auth.AuthInteractor
import com.jesuscb.footcoach.model.firebase.auth.IAuthInteractor
import com.jesuscb.footcoach.model.firebase.register.IRegisterInteractor
import com.jesuscb.footcoach.model.firebase.register.RegisterInteractor
import com.jesuscb.footcoach.model.firebase.exercise.ExerciseInteractor
import com.jesuscb.footcoach.model.firebase.exercise.IExerciseInteractor
import com.jesuscb.footcoach.model.firebase.match.IMatchInteractor
import com.jesuscb.footcoach.model.firebase.match.MatchInteractor
import com.jesuscb.footcoach.model.firebase.player.IPlayerInteractor
import com.jesuscb.footcoach.model.firebase.player.PlayerInteractor
import com.jesuscb.footcoach.model.firebase.training.ITrainingInteractor
import com.jesuscb.footcoach.model.firebase.training.TrainingInteractor

object FirebaseInteractor {

    private var mAuth: FirebaseAuth
    private var mDatabase: FirebaseDatabase
    private var mUsersReference: DatabaseReference
    private var initialised: Boolean = false

    var authInteractor: IAuthInteractor
    var registerInteractor: IRegisterInteractor
    lateinit var trainingInteractor: ITrainingInteractor
    lateinit var exerciseInteractor: IExerciseInteractor
    lateinit var matchInteractor: IMatchInteractor
    lateinit var playerInteractor: IPlayerInteractor

    //init function
    init {
        println("Getting Firebase Instances!!")
        mAuth = FirebaseAuth.getInstance()
        mDatabase = FirebaseDatabase.getInstance()
        mDatabase.setPersistenceEnabled(true)
        mUsersReference = mDatabase.reference.child("Users")
        println("Initialisig interactors!!")
        authInteractor = AuthInteractor()
        registerInteractor = RegisterInteractor()
        println("Interactors initilised!!")
    }

    // init de interactors of user
    fun initUsersInteractors() {
        if (!initialised) {
            println("Initialisig users interactors!!")
            trainingInteractor = TrainingInteractor()
            exerciseInteractor = ExerciseInteractor()
            playerInteractor = PlayerInteractor()
            matchInteractor = MatchInteractor()
            println("Users interactors initialised!!")
            initialised = true
        }
    }

    //ger auth instance
    fun getAuthInstance(): FirebaseAuth {
        return mAuth
    }

    //get database instance
    fun getDatabaseInstance(): FirebaseDatabase {
        return mDatabase
    }

    //get reference to branch Users
    fun getUsersReference(): DatabaseReference {
        return mUsersReference
    }

    //get UID of authenticated user
    fun getUserUID(): String {
        return authInteractor.getCurrentUser()!!.uid
    }

    // log out fron FirebaseAutentication
    fun singOut() {
        initialised=false
        authInteractor.signOut()
    }


}