package com.jesuscb.footcoach.model.firebase.exercise

import com.google.firebase.database.ValueEventListener
import com.jesuscb.footcoach.model.Exercise
import com.jesuscb.footcoach.model.firebase.core.FirebaseInteractor

class ExerciseInteractor : IExerciseInteractor {

    private var exercisesReference = FirebaseInteractor.getUsersReference().child(FirebaseInteractor.getUserUID()).child("exercises")

    //get list of exercises
    override fun getExercises(exercisesListener: ValueEventListener) {
        exercisesReference.addListenerForSingleValueEvent(exercisesListener)
    }

    //get list of specific ids exercises
    override fun getExercises(exercises: ArrayList<String>, exercisesListener: ValueEventListener) {
        for (id: String in exercises) {
            exercisesReference.child(id).addListenerForSingleValueEvent(exercisesListener)
        }
    }

    //get a exercise
    override fun getExercise(id: String, exerciseListener: ValueEventListener) {
        exercisesReference.child(id).addListenerForSingleValueEvent(exerciseListener)
    }

    //save a exercise
    override fun saveExercise(exercise: Exercise) {
        val key = exercisesReference.push().key
        exercise.id_exercise = key
        exercisesReference.child(key).setValue(exercise)
    }

    //update exercise
    override fun updateExercise(exercise: Exercise) {
        exercisesReference.child(exercise.id_exercise).setValue(exercise)
    }

    //save list of exercises
    override fun saveExercises(exercises: MutableList<Exercise>) {
        exercises.forEach {
            val key = exercisesReference.push().key
            it.id_exercise = key
            exercisesReference.child(key).setValue(it)
        }
    }

    //delete exercise
    override fun deleteExercise(id_exercise: String) {
        exercisesReference.child(id_exercise).removeValue()
    }

}