package com.jesuscb.footcoach.model.firebase.match

import com.google.firebase.database.ValueEventListener
import com.jesuscb.footcoach.model.Match
import com.jesuscb.footcoach.model.Player
import com.jesuscb.footcoach.model.firebase.core.FirebaseInteractor


class MatchInteractor : IMatchInteractor {

    private var matchsReference = FirebaseInteractor.getUsersReference().child(FirebaseInteractor.getUserUID()).child("matchs")

    override fun getMatchs(matchListener: ValueEventListener) {
        matchsReference.addListenerForSingleValueEvent(matchListener)
    }

    override fun getMatch(id: String, matchListener: ValueEventListener) {
        matchsReference.child(id).addListenerForSingleValueEvent(matchListener)
    }

    override fun saveMatch(match: Match): Match {
        val key = matchsReference.push().key
        match.id_match = key
        matchsReference.child(key).setValue(match)
        return match
    }

    override fun updateMatch(match: Match) {
        matchsReference.child(match.id_match).setValue(match)
    }

    override fun deletePlayerFromMatch(playerId: String, matchId: String) {
        matchsReference.child(matchId).child("/ids_players").child(playerId).removeValue()
        FirebaseInteractor.getUsersReference().child(FirebaseInteractor.getUserUID()).child("/players").child(playerId).child("/ids_matchs").child(matchId).removeValue()
    }

    override fun addSelectedPlayersToMatch(selectedPlayers : ArrayList<Player>, matchId: String) {
        val selectedPlayersIds : HashMap<String,Boolean> = hashMapOf()
        for(player in selectedPlayers){
            selectedPlayersIds.put(player.id_player,true)
        }

        //update ids_players from match
        val childUpdates: HashMap<String, Any> = hashMapOf()
        childUpdates.put("/matchs/" + matchId + "/ids_players", selectedPlayersIds);

        var match: HashMap<String, Boolean> = hashMapOf()

        //update training from players
        for (player in selectedPlayers) {
            player.ids_matchs.toMap(match)
            match.put(matchId, true)
            childUpdates.put("/players/" + player.id_player + "/ids_matchs", match);
        }

        FirebaseInteractor.getUsersReference().child(FirebaseInteractor.getUserUID()).updateChildren(childUpdates);
        //matchsReference.child(matchId).child("/ids_players").setValue(playersMap)
    }

    override fun deleteMatch(match: Match) {
        for(player in match.ids_players){
            FirebaseInteractor.getUsersReference().child(FirebaseInteractor.getUserUID()).child("/players").child(player.key).child("/ids_matchs").child(match.id_match).removeValue()
        }
        matchsReference.child(match.id_match).removeValue()
    }
}