package com.jesuscb.footcoach.model.firebase.training

import com.google.firebase.database.ValueEventListener
import com.jesuscb.footcoach.model.Exercise
import com.jesuscb.footcoach.model.Player
import com.jesuscb.footcoach.model.Training
import com.jesuscb.footcoach.model.firebase.core.FirebaseInteractor

/**
 * Access to trainings DB
 */
class TrainingInteractor : ITrainingInteractor {

    private var trainingsReference = FirebaseInteractor.getUsersReference().child(FirebaseInteractor.getUserUID()).child("trainings")

    //get trainings
    override fun getTrainings(trainingListener: ValueEventListener) {
        trainingsReference.addListenerForSingleValueEvent(trainingListener)
    }

    //get a training
    override fun getTraining(id: String, trainingListener: ValueEventListener) {
        trainingsReference.child(id).addListenerForSingleValueEvent(trainingListener)
    }

    //save a training
    override fun saveTraining(training: Training): Training {
        val key = trainingsReference.push().key
        training.id_training = key
        trainingsReference.child(key).setValue(training)
        return training
    }

    // update a training
    override fun updateTraining(training: Training) {
        trainingsReference.child(training.id_training).setValue(training)
    }

    //delete exercise from training
    override fun deleteExerciseFromTraining(exerciseId: String, trainingId: String) {
        trainingsReference.child(trainingId).child("/ids_exercises").child(exerciseId).removeValue()
        FirebaseInteractor.getUsersReference().child(FirebaseInteractor.getUserUID()).child("/exercises").child(exerciseId).child("/ids_trainings").child(trainingId).removeValue()
    }

    //delete a player from a training
    override fun deletePlayerFromTraining(playerId: String, trainingId: String) {
        trainingsReference.child(trainingId).child("/ids_players").child(playerId).removeValue()
        FirebaseInteractor.getUsersReference().child(FirebaseInteractor.getUserUID()).child("/players").child(playerId).child("/ids_trainings").child(trainingId).removeValue()

    }

    //add selected exercises to training
    override fun addSelectedExercisesToTraining(selectedExercises : ArrayList<Exercise>, trainingId: String) {
        val selectedExercisesIds : HashMap<String,Boolean> = hashMapOf()
        for(exercise in selectedExercises){
            selectedExercisesIds.put(exercise.id_exercise,true)
        }

        //update ids_exercises from training
        val childUpdates: HashMap<String, Any> = hashMapOf()
        childUpdates.put("/trainings/" + trainingId + "/ids_exercises", selectedExercisesIds);

        var training: HashMap<String, Boolean> = hashMapOf()

        //update training from players
        for (exercise in selectedExercises) {
            exercise.ids_trainings.toMap(training)
            training.put(trainingId, true)
            childUpdates.put("/exercises/" + exercise.id_exercise + "/ids_trainings", training);
        }

        FirebaseInteractor.getUsersReference().child(FirebaseInteractor.getUserUID()).updateChildren(childUpdates);

        //trainingsReference.child(trainingId).child("/ids_players").setValue(playersMap)
        //trainingsReference.child(trainingId).child("/ids_exercises").setValue(exercisesMap)
    }

    //add selected players to training
    override fun addSelectedPlayersToTraining(selectedPlayers: ArrayList<Player>, trainingId: String) {
        val selectedPlayersIds : HashMap<String,Boolean> = hashMapOf()
        for(player in selectedPlayers){
            selectedPlayersIds.put(player.id_player,true)
        }

        //update ids_players from training
        val childUpdates: HashMap<String, Any> = hashMapOf()
        childUpdates.put("/trainings/" + trainingId + "/ids_players", selectedPlayersIds);

        var training: HashMap<String, Boolean> = hashMapOf()

        //update training from players
        for (player in selectedPlayers) {
            player.ids_trainings.toMap(training)
            training.put(trainingId, true)
            childUpdates.put("/players/" + player.id_player + "/ids_trainings", training);
        }

        FirebaseInteractor.getUsersReference().child(FirebaseInteractor.getUserUID()).updateChildren(childUpdates);

        //trainingsReference.child(trainingId).child("/ids_players").setValue(playersMap)
    }

    //delete a training
    override fun deleteTraining(training: Training) {
        for(exercise in training.ids_exercises){
            FirebaseInteractor.getUsersReference().child(FirebaseInteractor.getUserUID()).child("/exercises").child(exercise.key).child("/ids_trainings").child(training.id_training).removeValue()
        }
        for(player in training.ids_players){
            FirebaseInteractor.getUsersReference().child(FirebaseInteractor.getUserUID()).child("/players").child(player.key).child("/ids_trainings").child(training.id_training).removeValue()
        }
        trainingsReference.child(training.id_training).removeValue()
    }
}