package com.jesuscb.footcoach.presenter.exercise

import com.google.firebase.database.DataSnapshot
import com.google.firebase.database.DatabaseError
import com.google.firebase.database.ValueEventListener
import com.jesuscb.footcoach.model.Exercise
import com.jesuscb.footcoach.model.firebase.core.FirebaseInteractor
import com.jesuscb.footcoach.model.firebase.exercise.IExerciseInteractor
import com.jesuscb.footcoach.view.exercise.ExerciseCRUDView

class ExerciseCRUDPresenter(val exerciseCRUDView: ExerciseCRUDView) {

    private var exerciseInteractor : IExerciseInteractor = FirebaseInteractor.exerciseInteractor
    private lateinit var exerciseListener: ValueEventListener

    fun start() {
        exerciseCRUDView.initialise()
    }

    fun addOrUpdateExercise(exercise: Exercise) {
        if (exercise.id_exercise.isNullOrBlank()) {
            exerciseInteractor.saveExercise(exercise)
        } else {
            exerciseInteractor.updateExercise(exercise)
        }
        exerciseCRUDView.exerciseModified()
    }

    fun getExercise(id: String) {
        createExerciseListener()
        // exerciseDetailView.showProgressBar()
        exerciseInteractor.getExercise(id, exerciseListener)
    }

    //Crea el listener para los ejercicios
    private fun createExerciseListener() {
        exerciseListener = object : ValueEventListener {
            override fun onDataChange(dataSnapshot: DataSnapshot) {
                var exercise: Exercise? = dataSnapshot.getValue<Exercise>(Exercise::class.java)
                exerciseCRUDView.updateDetailsExercise(exercise!!)
                //exerciseDetailView.hideProgressBar()
            }

            override fun onCancelled(databaseError: DatabaseError) {
                println("loadExercises:onCancelled ${databaseError.toException()}")
            }
        }
    }
}
