package com.jesuscb.footcoach.presenter.exercise

import com.google.firebase.database.DataSnapshot
import com.google.firebase.database.DatabaseError
import com.google.firebase.database.ValueEventListener
import com.jesuscb.footcoach.model.firebase.core.FirebaseInteractor
import com.jesuscb.footcoach.model.firebase.exercise.IExerciseInteractor
import com.jesuscb.footcoach.model.Exercise
import com.jesuscb.footcoach.model.Player
import com.jesuscb.footcoach.model.firebase.training.ITrainingInteractor
import com.jesuscb.footcoach.view.exercise.ExerciseListView

class ExerciseListPresenter(val exerciseView: ExerciseListView){

    private var trainingInteractor : ITrainingInteractor = FirebaseInteractor.trainingInteractor

    private var exerciseInteractor : IExerciseInteractor = FirebaseInteractor.exerciseInteractor
    private var exercises: MutableList<Exercise> = mutableListOf()
    private var exercisesRestricted: ArrayList<String> = arrayListOf()
    private lateinit var exerciseListener : ValueEventListener
    private lateinit var exerciseListenerFromId : ValueEventListener
    private lateinit var exerciseListenerRestricted : ValueEventListener

    fun start(){
        exerciseView.initialise()
        createExerciseListener()
    }

    fun startExerciseListTrainingDetail(){
        exerciseView.initialiseExerciseListTrainingDetail()
        createExerciseFromIdListener()
    }

    fun startExerciseAddToTrainingAdapter(exercisesToIgnore: ArrayList<String>?){
        exercisesRestricted=exercisesToIgnore!!
        exerciseView.initialiseExerciseListAddToTraining()
        createExerciseRestrictedListener()
    }

    fun getExercises(){
        exerciseView.showRefreshProgressBar()
        exerciseInteractor.getExercises(exerciseListener)
    }

    fun getExercisesFromIds(exerciseIds : ArrayList<String>){
        exerciseInteractor.getExercises(exerciseIds,exerciseListenerFromId)
    }

    fun getExercisesRestricted() {
        exerciseInteractor.getExercises(exerciseListenerRestricted)
    }

    fun deleteExercise(exercise:Exercise) {
        if(exercise.ids_trainings.isEmpty()){
            exerciseInteractor.deleteExercise(exercise.id_exercise)
            exerciseView.showSuccessDeletingExercise()
            getExercises()
        }else{
            exerciseView.showErrorDeletingExercise()
        }
    }

    //Crea el listener para los ejercicios
    private fun createExerciseListener() {
        exerciseListener = object : ValueEventListener {
            override fun onDataChange(dataSnapshot: DataSnapshot) {
                exercises.clear()
                dataSnapshot.children.mapNotNullTo(exercises) {
                    it.getValue<Exercise>(Exercise::class.java)
                }
                exerciseView.updateList(exercises)
                exerciseView.hideRefreshProgressBar()
            }

            override fun onCancelled(databaseError: DatabaseError) {
                println("loadExercises:onCancelled ${databaseError.toException()}")
            }
        }
    }

    private fun createExerciseFromIdListener() {
        exerciseListenerFromId = object : ValueEventListener {
            override fun onDataChange(dataSnapshot: DataSnapshot) {
                var exercise: Exercise? = dataSnapshot.getValue<Exercise>(Exercise::class.java)
                exerciseView.addItemToList(exercise!!)
            }

            override fun onCancelled(databaseError: DatabaseError) {
                println("loadExercise:onCancelled ${databaseError.toException()}")
            }
        }
    }

    //Crea el listener para los ejercicios
    private fun createExerciseRestrictedListener() {
        exerciseListenerRestricted = object : ValueEventListener {
            override fun onDataChange(dataSnapshot: DataSnapshot) {
                exercises.clear()
                dataSnapshot.children.mapNotNullTo(exercises) {
                    it.getValue<Exercise>(Exercise::class.java)
                }
                val exercisesToIgnore : ArrayList<Exercise> = ArrayList(exercises.filter { it -> exercisesRestricted.contains(it.id_exercise) })
                exerciseView.informExercisesIgnored(exercisesToIgnore)
                exercises.removeAll { it -> exercisesRestricted.contains(it.id_exercise) }
                exerciseView.updateList(exercises)
            }

            override fun onCancelled(databaseError: DatabaseError) {
                println("loadExercises:onCancelled ${databaseError.toException()}")
            }
        }
    }
}