package com.jesuscb.footcoach.presenter.match

import com.google.firebase.database.DataSnapshot
import com.google.firebase.database.DatabaseError
import com.google.firebase.database.ValueEventListener
import com.jesuscb.footcoach.model.Match
import com.jesuscb.footcoach.model.Player
import com.jesuscb.footcoach.model.firebase.core.FirebaseInteractor
import com.jesuscb.footcoach.model.firebase.match.IMatchInteractor
import com.jesuscb.footcoach.view.match.MatchDetailView

class MatchDetailPresenter(val matchDetailView: MatchDetailView) {
    private var matchInteractor: IMatchInteractor = FirebaseInteractor.matchInteractor
    private lateinit var matchListener: ValueEventListener

    fun start() {
        matchDetailView.initialise()
    }

    fun getMatch(id: String) {
        createMatchListener()
        matchInteractor.getMatch(id, matchListener)
    }

    fun deletePlayerFromMatch(playerId: String, matchId: String) {
        matchInteractor.deletePlayerFromMatch(playerId, matchId)
    }

    fun addSelectedPlayers(selectedPlayers: ArrayList<Player>, matchId: String) {
        matchInteractor.addSelectedPlayersToMatch(selectedPlayers, matchId)
        matchDetailView.showToastSuccessAddPlayersToMatchAndRefresh()
    }

    fun deleteMatch(match: Match) {
        matchInteractor.deleteMatch(match)
        matchDetailView.matchDeleted()
    }

    //Crea el listener para los partidos
    private fun createMatchListener() {
        matchListener = object : ValueEventListener {
            override fun onDataChange(dataSnapshot: DataSnapshot) {
                var match: Match? = dataSnapshot.getValue<Match>(Match::class.java)
                matchDetailView.updateDetailsMatch(match!!)
                matchDetailView.updatePlayerList(ArrayList(match.ids_players.keys))
                //exerciseDetailView.hideProgressBar()
            }

            override fun onCancelled(databaseError: DatabaseError) {
                println("loadTrainig:onCancelled ${databaseError.toException()}")
            }
        }
    }
}