package com.jesuscb.footcoach.presenter.match

import com.google.firebase.database.DataSnapshot
import com.google.firebase.database.DatabaseError
import com.google.firebase.database.ValueEventListener
import com.jesuscb.footcoach.model.Match
import com.jesuscb.footcoach.model.firebase.core.FirebaseInteractor
import com.jesuscb.footcoach.model.firebase.match.IMatchInteractor
import com.jesuscb.footcoach.view.match.MatchListView
import org.joda.time.LocalDateTime
import org.joda.time.format.DateTimeFormat


class MatchListPresenter(val matchListView: MatchListView) {

    private var matchInteractor: IMatchInteractor = FirebaseInteractor.matchInteractor
    private var matchs: MutableList<Match> = mutableListOf()
    private lateinit var matchListener: ValueEventListener

    fun start() {
        matchListView.initialise()
        createMatchListener()
    }

    fun getMatchs() {
        matchListView.showRefreshProgressBar()
        matchInteractor.getMatchs(matchListener)
    }

    //Crea el listener para los partidos
    private fun createMatchListener() {
        matchListener = object : ValueEventListener {
            override fun onDataChange(dataSnapshot: DataSnapshot) {
                matchs.clear()
                dataSnapshot.children.mapNotNullTo(matchs) {
                    it.getValue<Match>(Match::class.java)
                }
                matchListView.updateList(matchs)
                matchListView.hideRefreshProgressBar()
            }

            override fun onCancelled(databaseError: DatabaseError) {
                println("loadMatchs:onCancelled ${databaseError.toException()}")
            }
        }
    }
}