package com.jesuscb.footcoach.presenter.player

import com.google.firebase.database.DataSnapshot
import com.google.firebase.database.DatabaseError
import com.google.firebase.database.ValueEventListener
import com.jesuscb.footcoach.model.Match
import com.jesuscb.footcoach.model.Player
import com.jesuscb.footcoach.model.Training
import com.jesuscb.footcoach.model.firebase.core.FirebaseInteractor
import com.jesuscb.footcoach.model.firebase.match.IMatchInteractor
import com.jesuscb.footcoach.model.firebase.player.IPlayerInteractor
import com.jesuscb.footcoach.model.firebase.training.ITrainingInteractor
import com.jesuscb.footcoach.view.player.PlayerCRUDView

class PlayerCRUDPresenter(val playerCRUDView: PlayerCRUDView) {

    private var playerInteractor: IPlayerInteractor = FirebaseInteractor.playerInteractor
    private var matchInteractor: IMatchInteractor = FirebaseInteractor.matchInteractor
    private var trainingInteractor: ITrainingInteractor = FirebaseInteractor.trainingInteractor
    private lateinit var playerListener: ValueEventListener
    private lateinit var trainingListener: ValueEventListener
    private var trainings: ArrayList<Training> = arrayListOf()
    private lateinit var matchListener: ValueEventListener
    private var matchs: ArrayList<Match> = arrayListOf()


    fun start() {
        playerCRUDView.initialise()
    }

    fun addOrUpdatePlayer(player: Player) {
        if (player.id_player.isNullOrBlank()) {
            playerInteractor.savePlayer(player)
        } else {
            playerInteractor.updatePlayer(player)
        }
        playerCRUDView.playerModified()
    }

    fun getPlayer(id: String) {
        createPlayerListener()
        playerInteractor.getPlayer(id, playerListener)
    }

    //Crea el listener para los jugadores
    private fun createPlayerListener() {
        playerListener = object : ValueEventListener {
            override fun onDataChange(dataSnapshot: DataSnapshot) {
                var player: Player? = dataSnapshot.getValue<Player>(Player::class.java)
                playerCRUDView.updateDetailsPlayer(player!!)
                createMatchListener()
                matchInteractor.getMatchs(matchListener)
                createTrainingListener()
                trainingInteractor.getTrainings(trainingListener)
            }

            override fun onCancelled(databaseError: DatabaseError) {
                println("loadExercises:onCancelled ${databaseError.toException()}")
            }
        }
    }

    //Crea el listener para los entrenamientos
    private fun createTrainingListener() {
        trainingListener = object : ValueEventListener {
            override fun onDataChange(dataSnapshot: DataSnapshot) {
                trainings.clear()
                dataSnapshot.children.mapNotNullTo(trainings) {
                    it.getValue<Training>(Training::class.java)
                }
                playerCRUDView.updateDetailsPlayerTrainings(trainings)
            }

            override fun onCancelled(databaseError: DatabaseError) {
                println("loadTrainings:onCancelled ${databaseError.toException()}")
            }
        }
    }

    //Crea el listener para los partidos
    private fun createMatchListener() {
        matchListener = object : ValueEventListener {
            override fun onDataChange(dataSnapshot: DataSnapshot) {
                matchs.clear()
                dataSnapshot.children.mapNotNullTo(matchs) {
                    it.getValue<Match>(Match::class.java)
                }
                playerCRUDView.updateDetailsPlayerMatchs(matchs)
            }

            override fun onCancelled(databaseError: DatabaseError) {
                println("loadMatchs:onCancelled ${databaseError.toException()}")
            }
        }
    }
}
