package com.jesuscb.footcoach.presenter.player

import com.google.firebase.database.DataSnapshot
import com.google.firebase.database.DatabaseError
import com.google.firebase.database.ValueEventListener
import com.jesuscb.footcoach.model.Player
import com.jesuscb.footcoach.model.firebase.core.FirebaseInteractor
import com.jesuscb.footcoach.model.firebase.player.IPlayerInteractor
import com.jesuscb.footcoach.view.player.PlayerListView

class PlayerListPresenter(val playerView: PlayerListView) {

    private var playerInteractor: IPlayerInteractor = FirebaseInteractor.playerInteractor
    private var players: MutableList<Player> = mutableListOf()
    private var playersRestricted: ArrayList<String> = arrayListOf()
    private lateinit var playerListener: ValueEventListener
    private lateinit var playerListenerFromId: ValueEventListener
    private lateinit var playerListenerRestricted: ValueEventListener


    fun start() {
        playerView.initialise()
        createPlayerListener()
    }

    fun startPlayerListTrainingDetail() {
        playerView.initialisePlayerListTrainingDetail()
        createPlayerFromIdListener()
    }

    fun startPlayerAddToTrainingAdapter(playersToIgnore: ArrayList<String>?) {
        playersRestricted = playersToIgnore!!
        playerView.initialisePlayerListAddToTraining()
        createPlayerRestrictedListener()
    }

    fun getPlayers() {
        playerView.showRefreshProgressBar()
        playerInteractor.getPlayers(playerListener)
    }

    fun getPlayersFromIds(playerIds: ArrayList<String>) {
        playerInteractor.getPlayers(playerIds, playerListenerFromId)
    }

    fun getPlayersRestricted() {
        playerInteractor.getPlayers(playerListenerRestricted)
    }

    fun deletePlayer(player: Player) {
        if (player.ids_trainings.isEmpty() && player.ids_matchs.isEmpty()) {
            playerInteractor.deletePlayer(player.id_player)
            playerView.showSuccessDeletingPlayer()
            getPlayers()
        } else {
            playerView.showErrorDeletingPlayer()
        }
    }

    //Crea el listener para los jugadores
    private fun createPlayerListener() {
        playerListener = object : ValueEventListener {
            override fun onDataChange(dataSnapshot: DataSnapshot) {
                players.clear()
                dataSnapshot.children.mapNotNullTo(players) {
                    it.getValue<Player>(Player::class.java)
                }
                playerView.updateList(players)
                playerView.hideRefreshProgressBar()
            }

            override fun onCancelled(databaseError: DatabaseError) {
                println("loadPlayers:onCancelled ${databaseError.toException()}")
            }
        }
    }

    private fun createPlayerFromIdListener() {
        playerListenerFromId = object : ValueEventListener {
            override fun onDataChange(dataSnapshot: DataSnapshot) {
                val player: Player? = dataSnapshot.getValue<Player>(Player::class.java)
                playerView.addItemToList(player!!)
            }

            override fun onCancelled(databaseError: DatabaseError) {
                println("loadPlayer:onCancelled ${databaseError.toException()}")
            }
        }
    }

    //Crea el listener para los jugadores
    private fun createPlayerRestrictedListener() {
        playerListenerRestricted = object : ValueEventListener {
            override fun onDataChange(dataSnapshot: DataSnapshot) {
                players.clear()
                dataSnapshot.children.mapNotNullTo(players) {
                    it.getValue<Player>(Player::class.java)
                }
                val playersToIgnore: ArrayList<Player> = ArrayList(players.filter { it -> playersRestricted.contains(it.id_player) })
                playerView.informPlayersIgnored(playersToIgnore)
                players.removeAll { it -> playersRestricted.contains(it.id_player) }
                playerView.updateList(players)

            }

            override fun onCancelled(databaseError: DatabaseError) {
                println("loadPlayers:onCancelled ${databaseError.toException()}")
            }
        }
    }
}