package com.jesuscb.footcoach.presenter.training

import com.google.firebase.database.DataSnapshot
import com.google.firebase.database.DatabaseError
import com.google.firebase.database.ValueEventListener
import com.jesuscb.footcoach.model.Training
import com.jesuscb.footcoach.model.firebase.core.FirebaseInteractor
import com.jesuscb.footcoach.model.firebase.training.ITrainingInteractor
import com.jesuscb.footcoach.view.training.TrainingCRUDView


class TrainingCRUDPresenter(val trainingCRUDView: TrainingCRUDView) {
    private var trainingInteractor: ITrainingInteractor = FirebaseInteractor.trainingInteractor
    private lateinit var trainingListener: ValueEventListener

    fun start() {
        trainingCRUDView.initialise()
    }

    fun addOrUpdateTraining(training: Training) {
        if (training.id_training.isNullOrBlank()) {
            trainingInteractor.saveTraining(training)
        } else {
            trainingInteractor.updateTraining(training)
        }
        trainingCRUDView.trainingModified()
    }

    fun getTraining(id: String) {
        createTrainingListener()
        trainingInteractor.getTraining(id, trainingListener)
    }

    //Crea el listener para los entrenamientos
    private fun createTrainingListener() {
        trainingListener = object : ValueEventListener {
            override fun onDataChange(dataSnapshot: DataSnapshot) {
                var training: Training? = dataSnapshot.getValue<Training>(Training::class.java)
                trainingCRUDView.updateDetailsTraining(training!!)
                //exerciseDetailView.hideProgressBar()
            }

            override fun onCancelled(databaseError: DatabaseError) {
                println("loadTrainig:onCancelled ${databaseError.toException()}")
            }
        }
    }
}