package com.jesuscb.footcoach.presenter.training

import com.google.firebase.database.DataSnapshot
import com.google.firebase.database.DatabaseError
import com.google.firebase.database.ValueEventListener
import com.jesuscb.footcoach.model.Exercise
import com.jesuscb.footcoach.model.Player
import com.jesuscb.footcoach.model.Training
import com.jesuscb.footcoach.model.firebase.core.FirebaseInteractor
import com.jesuscb.footcoach.model.firebase.training.ITrainingInteractor
import com.jesuscb.footcoach.view.training.TrainingDetailView

class TrainingDetailPresenter(val trainingDetailView: TrainingDetailView) {
    private var trainingInteractor: ITrainingInteractor = FirebaseInteractor.trainingInteractor
    private lateinit var trainingListener: ValueEventListener

    fun start() {
        trainingDetailView.initialise()
    }

    fun getTraining(id: String) {
        createTrainingListener()
        trainingInteractor.getTraining(id, trainingListener)
    }

    fun deleteExerciseFromTraining(exerciseId: String, trainingId: String) {
        trainingInteractor.deleteExerciseFromTraining(exerciseId, trainingId)
    }

    fun deletePlayerFromTraining(playerId: String, trainingId: String) {
        trainingInteractor.deletePlayerFromTraining(playerId, trainingId)
    }

    fun addSelectedExercises(selectedExercises: ArrayList<Exercise>, trainingId: String) {
        trainingInteractor.addSelectedExercisesToTraining(selectedExercises, trainingId)
        trainingDetailView.showToastSuccessAddExercisesToTrainingAndRefresh()
    }

    fun addSelectedPlayers(selectedPlayers: ArrayList<Player>, trainingId: String) {
        trainingInteractor.addSelectedPlayersToTraining(selectedPlayers, trainingId)
        trainingDetailView.showToastSuccessAddPlayersToTrainingAndRefresh()
    }

    fun deleteTraining(training: Training) {
        trainingInteractor.deleteTraining(training)
        trainingDetailView.trainingDeleted()
    }

    //Crea el listener para los entrenamientos
    private fun createTrainingListener() {
        trainingListener = object : ValueEventListener {
            override fun onDataChange(dataSnapshot: DataSnapshot) {
                var training: Training? = dataSnapshot.getValue<Training>(Training::class.java)
                trainingDetailView.updateDetailsTraining(training!!)
                trainingDetailView.updateDataFragments(ArrayList(training.ids_exercises.keys), ArrayList(training.ids_players.keys))
                //exerciseDetailView.hideProgressBar()
            }

            override fun onCancelled(databaseError: DatabaseError) {
                println("loadTrainig:onCancelled ${databaseError.toException()}")
            }
        }
    }
}