package com.jesuscb.footcoach.presenter.training

import com.google.firebase.database.DataSnapshot
import com.google.firebase.database.DatabaseError
import com.google.firebase.database.ValueEventListener
import com.jesuscb.footcoach.model.Exercise
import com.jesuscb.footcoach.model.Training
import com.jesuscb.footcoach.model.firebase.core.FirebaseInteractor
import com.jesuscb.footcoach.model.firebase.exercise.IExerciseInteractor
import com.jesuscb.footcoach.model.firebase.training.ITrainingInteractor
import com.jesuscb.footcoach.view.training.TrainingGeneratorView
import java.util.*
import kotlin.math.roundToInt

class TrainingGeneratorPresenter(val trainingGeneratorView: TrainingGeneratorView) {

    private var trainingInteractor: ITrainingInteractor = FirebaseInteractor.trainingInteractor

    private var exerciseInteractor: IExerciseInteractor = FirebaseInteractor.exerciseInteractor
    private lateinit var exerciseListener: ValueEventListener

    private lateinit var training: Training
    private var trainingDuration: Int = 0
    private lateinit var exercisesTypes: HashMap<String, Int>
    private var exerciseList: ArrayList<Exercise> = arrayListOf()

    fun start() {
        trainingGeneratorView.initialise()
        createExerciseListener()
    }

    fun generateTraining(training: Training, duration_training: Int, exercisesTypes: HashMap<String, Int>) {
        trainingGeneratorView.showProgressBar(true)
        this.training = training
        this.trainingDuration = duration_training
        this.exercisesTypes = exercisesTypes
        exerciseInteractor.getExercises(exerciseListener)
    }

    private fun addExercisesToTraining() {
        val exercisesToTraining: HashMap<String, Boolean> = HashMap()
        val random = Random()
        for ((key, value) in exercisesTypes) {
            val exercisesFiltered = exerciseList.filter { exercise -> exercise.type.equals(key) }
            val exercisesAdded = arrayListOf<Int>()
            val size: Int = exercisesFiltered.size
            var count: Int = 0
            var countAux: Int = 0
            var duration: Int = 0
            while (count < size && countAux<50) {
                countAux++
                val index: Int = random.nextInt(size)
                if (!exercisesAdded.contains(index)) {
                    val exercise: Exercise = exercisesFiltered.get(index)
                    if (duration + exercise.duration <= value + 5) {
                        duration=duration+exercise.duration.roundToInt()
                        exercisesAdded.add(index)
                        exercisesToTraining.put(exercise.id_exercise, true)
                    }
                    count++
                }
            }
        }
        training.ids_exercises=exercisesToTraining
        saveDataTraining()
    }

    fun saveDataTraining() {
        training=trainingInteractor.saveTraining(training)
        trainingGeneratorView.trainingGenerated(training.id_training)

    }

    private fun createExerciseListener() {
        exerciseListener = object : ValueEventListener {
            override fun onDataChange(dataSnapshot: DataSnapshot) {
                exerciseList.clear()
                dataSnapshot.children.mapNotNullTo(exerciseList) {
                    it.getValue<Exercise>(Exercise::class.java)
                }
                addExercisesToTraining()
            }

            override fun onCancelled(databaseError: DatabaseError) {
                println("loadExercises:onCancelled ${databaseError.toException()}")
            }
        }
    }
}