package com.jesuscb.footcoach.presenter.training

import com.google.firebase.database.DataSnapshot
import com.google.firebase.database.DatabaseError
import com.google.firebase.database.ValueEventListener
import com.jesuscb.footcoach.model.Training
import com.jesuscb.footcoach.model.firebase.core.FirebaseInteractor
import com.jesuscb.footcoach.model.firebase.training.ITrainingInteractor
import com.jesuscb.footcoach.view.training.TrainingListView
import org.joda.time.format.DateTimeFormat


class TrainingListPresenter(val trainingListView: TrainingListView) {

    private var trainingInteractor: ITrainingInteractor = FirebaseInteractor.trainingInteractor
    private var trainings: MutableList<Training> = mutableListOf()
    private lateinit var trainingListener: ValueEventListener

    fun start() {
        trainingListView.initialise()
        createTrainingListener()
    }

    fun getTrainings() {
        trainingListView.showRefreshProgressBar()
        trainingInteractor.getTrainings(trainingListener)
    }

    //Crea el listener para los entrenamientos
    private fun createTrainingListener() {
        trainingListener = object : ValueEventListener {
            override fun onDataChange(dataSnapshot: DataSnapshot) {
                trainings.clear()
                dataSnapshot.children.mapNotNullTo(trainings) {
                    it.getValue<Training>(Training::class.java)
                }
                trainingListView.updateList(trainings)
                trainingListView.hideRefreshProgressBar()
            }

            override fun onCancelled(databaseError: DatabaseError) {
                println("loadTrainings:onCancelled ${databaseError.toException()}")
            }
        }
    }
}