package com.jesuscb.footcoach.view.exercise

import android.graphics.Color
import android.support.v7.widget.CardView
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import com.jesuscb.footcoach.R
import com.jesuscb.footcoach.model.Exercise


internal class ExerciseAdapter(var exerciseList: MutableList<Exercise>, val exerciseListFragment: ExerciseListFragment) : RecyclerView.Adapter<ExerciseAdapter.ExerciseViewHolder>() {

    override fun onBindViewHolder(holder: ExerciseViewHolder, position: Int) {
        holder.name.text = exerciseList[position].name
        holder.type.text = exerciseListFragment.getTypeValueFromKey(exerciseList[position].type)
        holder.duration.text = exerciseList[position].duration.toString().plus(" ").plus(exerciseListFragment.getString(R.string.text_minutes_min))

        holder.itemView.setOnClickListener { exerciseListFragment.viewDetailsExercise(exerciseList[position].id_exercise) }
        holder.itemView.setOnLongClickListener{exerciseListFragment.deleteExercise(exerciseList[position])}
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ExerciseViewHolder {
        val v = LayoutInflater.from(parent.context).inflate(R.layout.adapter_exercise, parent, false)
        if(viewType==0){
            v.findViewById<CardView>(R.id.card_exercise_list).setCardBackgroundColor(Color.parseColor("#E9FDFF"))
        }else{
            v.findViewById<CardView>(R.id.card_exercise_list).setCardBackgroundColor(Color.parseColor("#D7FCFF"))
        }
        return ExerciseViewHolder(v);
    }

    override fun getItemViewType(position: Int): Int {
        return position%2
    }

    override fun getItemCount(): Int {
        return exerciseList.size
    }

    internal class ExerciseViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        val name = itemView.findViewById<TextView>(R.id.name)
        val duration = itemView.findViewById<TextView>(R.id.duration)
        val type = itemView.findViewById<TextView>(R.id.type)

    }

    fun setItems(items: MutableList<Exercise>) {
        exerciseList = items
    }

    fun addItem(item: Exercise) {
        exerciseList.add(item)
    }
}