package com.jesuscb.footcoach.view.exercise

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.support.v7.app.AlertDialog
import android.support.v7.app.AppCompatActivity
import android.view.MenuItem
import com.basgeekball.awesomevalidation.AwesomeValidation
import com.basgeekball.awesomevalidation.ValidationStyle
import com.basgeekball.awesomevalidation.utility.custom.SimpleCustomValidation
import com.google.common.collect.Range
import com.jesuscb.footcoach.R
import com.jesuscb.footcoach.model.Exercise
import com.jesuscb.footcoach.presenter.exercise.ExerciseCRUDPresenter
import kotlinx.android.synthetic.main.activity_crud_exercise.*


class ExerciseCRUDActivity : AppCompatActivity(), ExerciseCRUDView {

    private val TAG = "ExerciseCRUDActivity"

    private lateinit var exerciseCRUDPresenter: ExerciseCRUDPresenter
    private lateinit var awesomeValidation: AwesomeValidation
    private var typeIndex: Int? = null
    private lateinit var exerciseId: String

    companion object {
        private val ARG_PARAM_EXERCISE_ID = "exerciseID"

        fun newIntent(context: Context, id: String): Intent {
            val intent = Intent(context, ExerciseCRUDActivity::class.java)
            intent.putExtra(ARG_PARAM_EXERCISE_ID, id)
            return intent
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_crud_exercise)
        setSupportActionBar(add_exercise_toolbar)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        exerciseId = intent.getStringExtra(ARG_PARAM_EXERCISE_ID)

        exerciseCRUDPresenter = ExerciseCRUDPresenter(this)
        exerciseCRUDPresenter.start()
    }

    override fun initialise() {
        awesomeValidation = AwesomeValidation(ValidationStyle.BASIC);
        setValidators()
        add_exercise_button.setOnClickListener { validateExercise() }
        type.setOnClickListener { createSingleSelectDialog(resources.getStringArray(R.array.exercise_type_values)) }
        if (!exerciseId.isNullOrBlank()) {
            supportActionBar!!.title=getString(R.string.text_details_exercise)
            exerciseCRUDPresenter.getExercise(exerciseId)
        }
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.getItemId()) {
            android.R.id.home -> exerciseNotModified()
        }
        return true
    }

    override fun updateDetailsExercise(exercise: Exercise) {
        name.setText(exercise.name)
        description.setText(exercise.description)
        duration.setText(exercise.duration.toString())
        duration_dist.setText(exercise.duration_dist)
        num_players.setText(exercise.num_players.toString())
        num_players_dist.setText(exercise.players_dist)
        material.setText(exercise.material)
        type.setText(getTypeValueFromKey(exercise.type))
    }

    override fun hideProgressBar() {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun showProgressBar() {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun exerciseModified() {
        val returnIntent = Intent()
        setResult(Activity.RESULT_OK, returnIntent)
        finish()
    }

    override fun exerciseNotModified() {
        val returnIntent = Intent()
        setResult(Activity.RESULT_CANCELED, returnIntent)
        finish()
    }

    private fun validateExercise() {
        if (awesomeValidation.validate()) {
            var exercise: Exercise = Exercise(name.text.toString(),
                    description.text.toString(),
                    duration.text.toString().toFloat(),
                    duration_dist.text.toString(),
                    num_players.text.toString().toInt(),
                    num_players_dist.text.toString(),
                    material.text.toString(),
                    resources.getStringArray(R.array.exercise_type_keys)[typeIndex!!])
            if(!exerciseId.isNullOrBlank()){
                exercise.id_exercise=exerciseId
            }
            exerciseCRUDPresenter.addOrUpdateExercise(exercise)
        }
    }

    private fun setValidators() {
        awesomeValidation.addValidation(this, name.id, SimpleCustomValidation { !name.text.isNullOrBlank() }, R.string.error_validation_required)
        awesomeValidation.addValidation(this, type.id, SimpleCustomValidation { typeIndex != null }, R.string.error_validation_required)
        awesomeValidation.addValidation(this, description.id, SimpleCustomValidation { !description.text.isNullOrBlank() }, R.string.error_validation_required)
        awesomeValidation.addValidation(this, duration.id, SimpleCustomValidation { !duration.text.isNullOrBlank() }, R.string.error_validation_required)
        awesomeValidation.addValidation(this, duration.id, Range.closed(0.5, 30.0), R.string.error_validation_duration_exercise_range)
        awesomeValidation.addValidation(this, num_players.id, SimpleCustomValidation { !num_players.text.isNullOrBlank() }, R.string.error_validation_required)
        awesomeValidation.addValidation(this, num_players.id, Range.closed(1, 22), R.string.error_validation_num_players_exercise_range)
    }

    private fun createSingleSelectDialog(items: Array<String>) {
        val mBuilder = AlertDialog.Builder(this)
        mBuilder.setTitle(getString(R.string.text_choose_item))
        mBuilder.setSingleChoiceItems(items, -1) { dialogInterface, i ->
            type.setText(items[i])
            type.setError(null)
            typeIndex = i
            dialogInterface.dismiss()
        }
        mBuilder.setNeutralButton(getString(R.string.text_cancel)) { dialog, _ ->
            dialog.cancel()
        }
        val mDialog = mBuilder.create()
        mDialog.show()
    }

    fun getTypeValueFromKey(key: String): String {
        var value: String = ""
        typeIndex = resources.getStringArray(R.array.exercise_type_keys).indexOf(key)
        if ( typeIndex != -1) {
            value = resources.getStringArray(R.array.exercise_type_values)[typeIndex!!]
        }
        return value
    }
}
