package com.jesuscb.footcoach.view.exercise

import android.app.Activity.RESULT_CANCELED
import android.app.Activity.RESULT_OK
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.support.design.widget.FloatingActionButton
import android.support.v4.app.Fragment
import android.support.v4.widget.SwipeRefreshLayout
import android.support.v7.app.AlertDialog
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.LinearLayout
import android.widget.ProgressBar
import android.widget.Toast
import com.jesuscb.footcoach.R
import com.jesuscb.footcoach.model.Exercise
import com.jesuscb.footcoach.model.Player
import com.jesuscb.footcoach.presenter.exercise.ExerciseListPresenter
import com.jesuscb.footcoach.view.core.IExerciseListToActivity
import kotlinx.android.synthetic.main.fragment_list_exercises.*


class ExerciseListFragment : Fragment(), ExerciseListView {

    private lateinit var exercisesAdapter: ExerciseAdapter
    private lateinit var exercisesDetailTrainingAdapter: ExerciseDetailTrainingAdapter
    private lateinit var exercisesAddToTrainingAdapter: ExerciseAddToTrainingAdapter
    private lateinit var exerciseListPresenter: ExerciseListPresenter
    private lateinit var swipeRefreshLayout: SwipeRefreshLayout
    private lateinit var progressBar: ProgressBar
    private var method: Int? = null

    private var activityCallback: IExerciseListToActivity? = null

    private val TAG_ADD_EXERCISE_RESULT = 1
    private val TAG_DETAILS_EXERCISE_RESULT = 2

    private val TAG = "ExerciseListFragment"

    companion object {
        private val ARG_FRAGMENT_METHOD_ID = "methodID"
        private val ARG_FRAGMENT_EXERCISES_ID = "exercisesID"

        fun newInstance(method: Int): ExerciseListFragment {
            val args: Bundle = Bundle()
            args.putInt(ARG_FRAGMENT_METHOD_ID, method)
            val fragment = ExerciseListFragment()
            fragment.arguments = args
            return fragment
        }

        fun newInstanceSelectedExercises(method: Int,selectExercises:ArrayList<String>): ExerciseListFragment {
            val args: Bundle = Bundle()
            args.putInt(ARG_FRAGMENT_METHOD_ID, method)
            args.putStringArrayList(ARG_FRAGMENT_EXERCISES_ID, selectExercises)
            val fragment = ExerciseListFragment()
            fragment.arguments = args
            return fragment
        }
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        try {
            activityCallback = context as IExerciseListToActivity
        } catch (e: ClassCastException) {
            throw ClassCastException(context.toString() + " must implement IExerciseListToDetailActivity")
        }
    }

    override fun onDetach() {
        activityCallback = null
        super.onDetach()
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState);
        exerciseListPresenter = ExerciseListPresenter(this)
        if (arguments != null && arguments!!.containsKey(ARG_FRAGMENT_METHOD_ID)) {
            method = arguments!!.getInt(ARG_FRAGMENT_METHOD_ID)
            when (method) {
                1 -> {
                    exerciseListPresenter.startExerciseListTrainingDetail()
                }
                2 -> {
                    var exercisesToIgnore = arguments!!.getStringArrayList(ARG_FRAGMENT_EXERCISES_ID)
                    exerciseListPresenter.startExerciseAddToTrainingAdapter(exercisesToIgnore)
                }
            }
        } else {
            exerciseListPresenter.start()
        }
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        when (method) {
            1 -> {
                val view: View = inflater.inflate(R.layout.fragment_list_exercises_detail_training, container, false)

                val rv = view.findViewById<RecyclerView>(R.id.list_exercises)
                rv.layoutManager = LinearLayoutManager(context, LinearLayout.VERTICAL, false)
                rv.adapter = exercisesDetailTrainingAdapter

                val btn: Button = view.findViewById<Button>(R.id.add_exercises_btn)
                btn.setOnClickListener { addExercisesToTraining() }

                exerciseListPresenter.getExercisesFromIds(arrayListOf())
                return view
            }
            2 -> {
                val view: View = inflater.inflate(R.layout.fragment_list_exercises_add_to_training, container, false)

                val rv = view.findViewById<RecyclerView>(R.id.list_exercises)
                rv.layoutManager = LinearLayoutManager(context, LinearLayout.VERTICAL, false)
                rv.adapter = exercisesAddToTrainingAdapter

                exerciseListPresenter.getExercisesRestricted()
                return view
            }
            else -> {
                val view: View = inflater.inflate(R.layout.fragment_list_exercises, container, false)

                val rv = view.findViewById<RecyclerView>(R.id.list_exercises)
                rv.layoutManager = LinearLayoutManager(context, LinearLayout.VERTICAL, false)
                rv.adapter = exercisesAdapter

                val fab: FloatingActionButton = view.findViewById<FloatingActionButton>(R.id.fab_add_exercise)
                fab.setOnClickListener { newExercise() }

                swipeRefreshLayout = view.findViewById<SwipeRefreshLayout>(R.id.swipe_list_exercises);
                progressBar = view.findViewById<ProgressBar>(R.id.progressBar_exercises);
                initSwipeToRefreshView()

                progressBar.visibility=View.VISIBLE
                exerciseListPresenter.getExercises()
                return view
            }
        }
    }

    override fun initialise() {
        exercisesAdapter = ExerciseAdapter(mutableListOf(), this)
    }

    override fun initialiseExerciseListTrainingDetail() {
        exercisesDetailTrainingAdapter = ExerciseDetailTrainingAdapter(mutableListOf(), this)
    }

    override fun initialiseExerciseListAddToTraining() {
        exercisesAddToTrainingAdapter = ExerciseAddToTrainingAdapter(mutableListOf(), this)
    }

    override fun updateList(exerciseList: MutableList<Exercise>) {
        when (method) {
            2 -> {
                exercisesAddToTrainingAdapter.setItems(exerciseList)
                exercisesAddToTrainingAdapter.notifyDataSetChanged()
            }
            else -> {
                exercisesAdapter.setItems(exerciseList)
                exercisesAdapter.notifyDataSetChanged()
                if(exerciseList.isEmpty()){
                    emptyState.visibility=View.VISIBLE
                }else{
                    emptyState.visibility=View.GONE
                }
            }
        }
    }

    override fun addItemToList(exercise: Exercise) {
        exercisesDetailTrainingAdapter.addItem(exercise)
        exercisesDetailTrainingAdapter.notifyDataSetChanged()
    }

    override fun informExercisesIgnored(exercisesToIgnore: ArrayList<Exercise>) {
        activityCallback!!.informExercisesIgnored(exercisesToIgnore)
    }

    fun refreshList(exercises: ArrayList<String>) {
        exercisesDetailTrainingAdapter.exerciseList.clear()
        if (exercises.isEmpty()) {
            exercisesDetailTrainingAdapter.notifyDataSetChanged()
        } else {
            exerciseListPresenter.getExercisesFromIds(exercises)
        }
    }

    private fun newExercise() {
        val intent = ExerciseCRUDActivity.newIntent(context!!, "")
        startActivityForResult(intent, TAG_ADD_EXERCISE_RESULT)
    }

    fun viewDetailsExercise(id: String) {
        val intent = ExerciseCRUDActivity.newIntent(context!!, id)
        startActivityForResult(intent, TAG_DETAILS_EXERCISE_RESULT)
    }

    fun deleteExercise(exercise: Exercise):Boolean{
        val mBuilder = AlertDialog.Builder(context!!)
        mBuilder.setTitle(getString(R.string.text_delete_exercise))

        mBuilder.setMessage(getString(R.string.text_delete_exercise_confirm_message))

        mBuilder.setPositiveButton(getString(R.string.text_yes)) { _, _ ->
            exerciseListPresenter.deleteExercise(exercise)
        }
        mBuilder.setNeutralButton(getString(R.string.text_cancel)) { dialog, _ ->
            dialog.cancel()
        }

        val mDialog = mBuilder.create()
        mDialog.show()
        return true
    }

    fun getExercisesFromAdapter():ArrayList<Exercise> {
       return exercisesAddToTrainingAdapter.getSelectedExercises()
    }

    fun deleteExerciseFromTraining(idExercise: String) {
        activityCallback!!.deleteExercise(idExercise)
    }

    private fun addExercisesToTraining() {
        activityCallback!!.addExercisesToTraining(exercisesDetailTrainingAdapter.exercisesIds())
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        when (requestCode) {
            TAG_ADD_EXERCISE_RESULT -> {
                when (resultCode) {
                    RESULT_OK -> {
                        Toast.makeText(context, getString(R.string.success_adding_exercise), Toast.LENGTH_SHORT).show()
                        exerciseListPresenter.getExercises()
                    }
                    RESULT_CANCELED -> {
                        Log.i(TAG, "Operación cancelada")
                    }
                    else -> {
                        showToastErrorAddExercise()
                    }
                }
            }
            TAG_DETAILS_EXERCISE_RESULT -> {
                when (resultCode) {
                    RESULT_OK -> {
                        Toast.makeText(context, getString(R.string.success_modified_exercise), Toast.LENGTH_SHORT).show()
                        exerciseListPresenter.getExercises()
                    }
                    RESULT_CANCELED -> {
                        Log.i(TAG, "Operación cancelada")
                    }
                    else -> {
                        showToastErrorUpdateExercise()
                    }
                }
            }
        }
    }

    private fun showToastErrorAddExercise() {
        Toast.makeText(context, getString(R.string.error_adding_exercise), Toast.LENGTH_SHORT).show()
    }

    private fun showToastErrorUpdateExercise() {
        Toast.makeText(context, getString(R.string.error_updating_exercise), Toast.LENGTH_SHORT).show()
    }

    override fun showErrorDeletingExercise() {
        Toast.makeText(context, getString(R.string.error_deleting_exercise), Toast.LENGTH_SHORT).show()
    }

    override fun showSuccessDeletingExercise() {
        Toast.makeText(context, getString(R.string.success_deleting_exercise), Toast.LENGTH_SHORT).show()
    }

    private fun initSwipeToRefreshView() {
        swipeRefreshLayout.setEnabled(true)
        swipeRefreshLayout.setOnRefreshListener(
                SwipeRefreshLayout.OnRefreshListener {
                    exerciseListPresenter.getExercises()
                }
        )
    }

    override fun showRefreshProgressBar() {
        if (!swipeRefreshLayout.isRefreshing()) {
            swipeRefreshLayout.post(Runnable {
                swipeRefreshLayout.setRefreshing(true)
                swipeRefreshLayout.setEnabled(false)
            })
        }
    }

    override fun hideRefreshProgressBar() {
        if (swipeRefreshLayout.isRefreshing()) {
            swipeRefreshLayout.post(Runnable {
                swipeRefreshLayout.setEnabled(true)
                swipeRefreshLayout.setRefreshing(false)
            })
        }
        progressBar.visibility = View.GONE
    }

    fun getTypeValueFromKey(key: String): String {
        var value: String = ""
        val typeIndex = resources.getStringArray(R.array.exercise_type_keys).indexOf(key)
        if (typeIndex != -1) {
            value = resources.getStringArray(R.array.exercise_type_values)[typeIndex]
        }
        return value
    }
}