package com.jesuscb.footcoach.view.login


import android.content.Intent
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.view.View
import android.widget.Toast
import com.basgeekball.awesomevalidation.AwesomeValidation
import com.basgeekball.awesomevalidation.ValidationStyle
import com.basgeekball.awesomevalidation.utility.custom.SimpleCustomValidation
import com.jesuscb.footcoach.view.main.MainActivity
import com.jesuscb.footcoach.R
import com.jesuscb.footcoach.presenter.LoginPresenter
import com.jesuscb.footcoach.view.register.RegisterActivity
import kotlinx.android.synthetic.main.activity_login.*


/**
 * A login screen that offers login via email/password.
 */

class LoginActivity :  AppCompatActivity(), LoginView {

    private val TAG = "LoginActivity"

    private lateinit var loginPresenter : LoginPresenter
    private lateinit var awesomeValidation: AwesomeValidation

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_login)

        loginPresenter = LoginPresenter(this)
        loginPresenter.start()
    }

    override fun initialise() {
        awesomeValidation = AwesomeValidation(ValidationStyle.BASIC);
        setValidators()
        login_button.setOnClickListener{loginUser()}
        register_link.setOnClickListener{loadRegisterForm()}
    }

    private fun setValidators() {
        awesomeValidation.addValidation(this, email.id, android.util.Patterns.EMAIL_ADDRESS, R.string.error_validation_email)
        awesomeValidation.addValidation(this, password.id, SimpleCustomValidation { password.text.toString().length>=6 }, R.string.error_validation_password)
    }

    private fun loginUser() {
        if (awesomeValidation.validate()) {
            loginPresenter.login(email.text.toString(),password.text.toString())
        }
    }

    override fun loginSucessfull() {
        val intent = Intent(this@LoginActivity, MainActivity::class.java)
        intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK)
        startActivity(intent)
    }

    private fun loadRegisterForm() {
        val intent = Intent(this@LoginActivity, RegisterActivity::class.java)
        intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK)
        startActivity(intent)
    }

    override fun showProgressBar(state: Boolean) {
       if (state){
           login_progress.visibility=View.VISIBLE
       }else{
           login_progress.visibility=View.GONE
       }
    }

    override fun showLoginErrorToast() {
        Toast.makeText(this, getString(R.string.error_validate_login_credentials), Toast.LENGTH_SHORT).show()
    }
}
