package com.jesuscb.footcoach.view.main

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.support.v4.app.Fragment
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.LinearLayout
import android.widget.ProgressBar
import android.widget.Toast
import com.jesuscb.footcoach.R
import com.jesuscb.footcoach.model.Training
import com.jesuscb.footcoach.presenter.training.TrainingListPresenter
import com.jesuscb.footcoach.view.core.Utils
import com.jesuscb.footcoach.view.training.TrainingAdapterHome
import com.jesuscb.footcoach.view.training.TrainingDetailActivity
import com.jesuscb.footcoach.view.training.TrainingGeneratorActivity
import com.jesuscb.footcoach.view.training.TrainingListView
import kotlinx.android.synthetic.main.fragment_home.*
import org.joda.time.DateTime


class HomeFragment : Fragment(), TrainingListView {

    private lateinit var trainingAdapter: TrainingAdapterHome
    private lateinit var trainingListPresenter: TrainingListPresenter
    private lateinit var progressBar: ProgressBar

    private val TAG_DETAILS_TRAINING_RESULT = 1

    private val TAG = "HomeFragment"

    @Override
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState);
        trainingListPresenter = TrainingListPresenter(this)
        trainingListPresenter.start()
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        // Inflate the layout for this fragment
        val view: View = inflater.inflate(R.layout.fragment_home, container, false)

        val gen_train_btn: Button = view.findViewById<Button>(R.id.generate_session_btn)
        gen_train_btn.setOnClickListener { generateTraining() }

        //RecyclerView
        val rv = view.findViewById<RecyclerView>(R.id.trainings_list_next)
        rv.layoutManager = LinearLayoutManager(context, LinearLayout.VERTICAL, false)
        rv.adapter = trainingAdapter

        progressBar = view.findViewById<ProgressBar>(R.id.progressBar_trainings_next);

        showRefreshProgressBar()
        //Get training list
        trainingListPresenter.getTrainings()
        return view
    }

    override fun initialise() {
        trainingAdapter = TrainingAdapterHome(mutableListOf(), this)
    }

    private fun generateTraining() {
        val intent = Intent(context, TrainingGeneratorActivity::class.java)
        startActivity(intent)
    }

    fun viewDetailsTraining(id: String) {
        val intent = TrainingDetailActivity.newIntent(context!!, id)
        startActivityForResult(intent, TAG_DETAILS_TRAINING_RESULT)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        when (requestCode) {
            TAG_DETAILS_TRAINING_RESULT -> {
                when (resultCode) {
                    Activity.RESULT_OK -> {
                        Toast.makeText(context, getString(R.string.success_deleting_training), Toast.LENGTH_SHORT).show()
                        trainingListPresenter.getTrainings()
                    }
                    Activity.RESULT_CANCELED -> {
                        trainingListPresenter.getTrainings()
                        Log.i(TAG, "Operación cancelada")
                    }
                    else -> {
                        Log.i(TAG, "Operación error")
                    }
                }
            }
        }
    }

    override fun showRefreshProgressBar() {
        progressBar.visibility = View.VISIBLE
    }

    override fun hideRefreshProgressBar() {
        progressBar.visibility = View.GONE
    }

    override fun updateList(trainingList: MutableList<Training>) {
        val trainingListOrdered = trainingList.sortedWith(Utils.CompareTrainingsDatesAscending).toMutableList()
        val trainingListOrderedFiltered = trainingListOrdered.filter { training -> Utils.parseStringToDateTimeHour(training.date,training.start_time).compareTo(DateTime.now()) >= 0 }.toMutableList()
        this.trainingAdapter.setItems(trainingListOrderedFiltered)
        this.trainingAdapter.notifyDataSetChanged()
        if (trainingListOrderedFiltered.isEmpty()) {
            emptyState.visibility = View.VISIBLE
            title_trainings_container.visibility = View.INVISIBLE
        } else {
            emptyState.visibility = View.GONE
            title_trainings_container.visibility = View.VISIBLE
        }
    }
}