package com.jesuscb.footcoach.view.main

import android.content.Intent
import android.os.Bundle
import android.support.design.widget.BottomNavigationView
import android.support.v7.app.AlertDialog
import android.util.Log
import android.view.Menu
import android.view.MenuItem
import android.widget.Toast
import com.jesuscb.footcoach.R
import com.jesuscb.footcoach.model.Exercise
import com.jesuscb.footcoach.model.Player
import com.jesuscb.footcoach.model.firebase.core.FirebaseInteractor
import com.jesuscb.footcoach.presenter.MainPresenter
import com.jesuscb.footcoach.view.core.BaseActivity
import com.jesuscb.footcoach.view.core.IExerciseListToActivity
import com.jesuscb.footcoach.view.core.IPlayerListToActivity
import com.jesuscb.footcoach.view.exercise.ExerciseListFragment
import com.jesuscb.footcoach.view.login.LoginActivity
import com.jesuscb.footcoach.view.match.MatchListFragment
import com.jesuscb.footcoach.view.player.PlayerListFragment
import com.jesuscb.footcoach.view.training.TrainingListFragment
import kotlinx.android.synthetic.main.activity_main.*


class MainActivity : BaseActivity(), MainView, IExerciseListToActivity, IPlayerListToActivity {

    private val TAG = "MainActivity"

    private val INTERVAL = 2000 //2 segundos para salir
    private var firstClickTime: Long = 0

    private lateinit var mainPresenter: MainPresenter

    lateinit var homeFragment: HomeFragment
    var matchsFragment: MatchListFragment? = null
    var exerciseFragment: ExerciseListFragment? = null
    var playerFragment: PlayerListFragment? = null
    var trainingFragment: TrainingListFragment? = null

    private val mOnNavigationItemSelectedListener = BottomNavigationView.OnNavigationItemSelectedListener { item ->
        when (item.itemId) {
            R.id.navigation_home -> {
                replaceFragment(homeFragment, R.id.fragment_main, getString(R.string.title_home))
                return@OnNavigationItemSelectedListener true
            }
            R.id.navigation_trainings -> {
                if (trainingFragment == null) trainingFragment = TrainingListFragment()
                replaceFragment(trainingFragment!!, R.id.fragment_main, getString(R.string.title_trainings))
                return@OnNavigationItemSelectedListener true
            }
            R.id.navigation_matchs -> {
                if (matchsFragment == null) matchsFragment = MatchListFragment()
                replaceFragment(matchsFragment!!, R.id.fragment_main, getString(R.string.title_matchs))
                return@OnNavigationItemSelectedListener true
            }
            R.id.navigation_players -> {
                if (playerFragment == null) playerFragment = PlayerListFragment()
                replaceFragment(playerFragment!!, R.id.fragment_main, getString(R.string.title_players))
                return@OnNavigationItemSelectedListener true
            }
            R.id.navigation_exercises -> {
                //exerciseFragment = ExerciseFragment.newInstance(exercises)
                if (exerciseFragment == null) exerciseFragment = ExerciseListFragment()
                replaceFragment(exerciseFragment!!, R.id.fragment_main, getString(R.string.title_exercises))
                return@OnNavigationItemSelectedListener true
            }
        }
        false
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)
        setSupportActionBar(app_toolbar)

        mainPresenter = MainPresenter(this)
        mainPresenter.start()
    }

    override fun initialise() {
        //Inicializamos el fragment principal
        homeFragment = HomeFragment()
        addFragment(homeFragment, R.id.fragment_main)

        navigation.setOnNavigationItemSelectedListener(mOnNavigationItemSelectedListener)
    }

    //Option menú
    override fun onCreateOptionsMenu(menu: Menu): Boolean {
        val inflater = menuInflater
        inflater.inflate(R.menu.toolbar_menu, menu)
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        // Handle item selection
        when (item.getItemId()) {
            R.id.item_sing_out -> {
                createConfirmationDialog()
                return true
            }
            else -> return super.onOptionsItemSelected(item)
        }
    }

    private fun singOutSucessfull() {
        val intent = Intent(this@MainActivity, LoginActivity::class.java)
        intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK)
        moveTaskToBack(true);
        finish()
        startActivity(intent)
    }

    private fun createConfirmationDialog() {
        val mBuilder = AlertDialog.Builder(this)
        mBuilder.setTitle(getString(R.string.text_sing_out_confirm_title))

        mBuilder.setMessage(getString(R.string.text_sing_out_confirm_message))

        mBuilder.setPositiveButton(getString(R.string.text_yes)) { _, _ ->
            FirebaseInteractor.singOut()
            singOutSucessfull()
        }
        mBuilder.setNeutralButton(getString(R.string.text_cancel)) { dialog, _ ->
            dialog.cancel()
        }

        val mDialog = mBuilder.create()
        mDialog.show()
    }

    override fun onBackPressed() {
        if (firstClickTime + INTERVAL > System.currentTimeMillis()) {
            moveTaskToBack(true); // exist app
            finish();
        } else {
            Toast.makeText(this, getString(R.string.text_info_exit_session), Toast.LENGTH_SHORT).show()
        }
        firstClickTime = System.currentTimeMillis()
    }

    override fun deleteExercise(exercise: String) {
        Log.i(TAG, "DeleteExercise no debería haber sido llamado")
    }

    override fun deletePlayer(player: String) {
        Log.i(TAG, "DeletePlayer no debería haber sido llamado")
    }

    override fun addExercisesToTraining(exercisesIds: ArrayList<String>) {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun addPlayersToActivity(players: ArrayList<String>) {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun informPlayersIgnored(players: ArrayList<Player>) {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun informExercisesIgnored(exercisesToIgnore: ArrayList<Exercise>) {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }
}
