package com.jesuscb.footcoach.view.match

import android.app.Activity
import android.app.DatePickerDialog
import android.app.TimePickerDialog
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.view.MenuItem
import android.widget.DatePicker
import android.widget.EditText
import android.widget.TimePicker
import com.basgeekball.awesomevalidation.AwesomeValidation
import com.basgeekball.awesomevalidation.ValidationStyle
import com.basgeekball.awesomevalidation.utility.custom.SimpleCustomValidation
import com.jesuscb.footcoach.R
import com.jesuscb.footcoach.model.Match
import com.jesuscb.footcoach.presenter.match.MatchCRUDPresenter
import kotlinx.android.synthetic.main.activity_crud_match.*
import java.text.SimpleDateFormat
import java.util.*

class MatchCRUDActivity : AppCompatActivity(), MatchCRUDView {

    private val TAG = "MatchCRUDActivity"

    private lateinit var matchCRUDPresenter: MatchCRUDPresenter
    private lateinit var awesomeValidation: AwesomeValidation
    private var cal: Calendar = Calendar.getInstance()
    private var dateSetListener: DatePickerDialog.OnDateSetListener? = null
    private var timeSetListener: TimePickerDialog.OnTimeSetListener? = null
    private lateinit var matchId: String
    private var match: Match? = null

    companion object {
        private val ARG_PARAM_MATCH_ID = "matchID"

        fun newIntent(context: Context, id: String): Intent {
            val intent = Intent(context, MatchCRUDActivity::class.java)
            intent.putExtra(ARG_PARAM_MATCH_ID, id)
            return intent
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_crud_match)
        setSupportActionBar(update_match_toolbar)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        matchId = intent.getStringExtra(MatchCRUDActivity.ARG_PARAM_MATCH_ID)

        matchCRUDPresenter = MatchCRUDPresenter(this)
        matchCRUDPresenter.start()
    }

    override fun initialise() {
        awesomeValidation = AwesomeValidation(ValidationStyle.BASIC);
        createDateSetListener()
        setValidators()
        day_match.setOnClickListener { showDatePickerDialog() }
        start_time.setOnClickListener { showTimePickerDialog(start_time) }
        end_time.setOnClickListener { showTimePickerDialog(end_time) }
        update_match_button.setOnClickListener { validateMatch() }
        if (!matchId.isNullOrBlank()) {
            supportActionBar!!.title = getString(R.string.text_update_match)
            matchCRUDPresenter.getMatch(matchId)
        }
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.getItemId()) {
            android.R.id.home -> matchNotModified()
        }
        return true
    }

    override fun updateDetailsMatch(match: Match) {
        day_match.setText(match.date)
        start_time.setText(match.start_time)
        end_time.setText(match.end_time)
        field_match.setText(match.field)
        rival_match.setText(match.rival)
        this.match = match
    }

    override fun hideProgressBar() {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun showProgressBar() {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun matchModified() {
        val returnIntent = Intent()
        setResult(Activity.RESULT_OK, returnIntent)
        finish()
    }

    override fun matchNotModified() {
        val returnIntent = Intent()
        setResult(Activity.RESULT_CANCELED, returnIntent)
        finish()
    }

    private fun validateMatch() {
        if (awesomeValidation.validate()) {
            if (match != null) {
                match!!.start_time = start_time.text.toString()
                match!!.end_time = end_time.text.toString()
                match!!.date = day_match.text.toString()
                match!!.rival = rival_match.text.toString()
                match!!.field = field_match.text.toString()
            } else {
                match = Match(day_match.text.toString(), start_time.text.toString(), end_time.text.toString(), field_match.text.toString(), rival_match.text.toString())
            }
            matchCRUDPresenter.addOrUpdateMatch(match!!)
        }
    }

    private fun showDatePickerDialog() {
        DatePickerDialog(this@MatchCRUDActivity,
                dateSetListener,
                cal.get(Calendar.YEAR),
                cal.get(Calendar.MONTH),
                cal.get(Calendar.DAY_OF_MONTH)).show()
    }

    private fun showTimePickerDialog(editTextField: EditText) {
        createTimeSetListener(editTextField)
        TimePickerDialog(this@MatchCRUDActivity,
                timeSetListener,
                cal.get(Calendar.HOUR_OF_DAY),
                cal.get(Calendar.MINUTE),
                true).show()
    }

    private fun createDateSetListener() {
        dateSetListener = object : DatePickerDialog.OnDateSetListener {
            override fun onDateSet(view: DatePicker, year: Int, monthOfYear: Int,
                                   dayOfMonth: Int) {
                cal.set(Calendar.YEAR, year)
                cal.set(Calendar.MONTH, monthOfYear)
                cal.set(Calendar.DAY_OF_MONTH, dayOfMonth)
                updateDateInView()
            }
        }
    }

    private fun createTimeSetListener(editTextField: EditText) {
        timeSetListener = object : TimePickerDialog.OnTimeSetListener {
            override fun onTimeSet(view: TimePicker?, hourOfDay: Int, minute: Int) {
                cal.set(Calendar.HOUR_OF_DAY, hourOfDay)
                cal.set(Calendar.MINUTE, minute)
                updateTimeInView(editTextField)
            }
        }
    }

    private fun updateDateInView() {
        val myFormat = "dd/MM/yyyy"
        val sdf = SimpleDateFormat(myFormat, Locale.US)
        day_match.setText(sdf.format(cal.getTime()))
        day_match.setError(null)
    }

    private fun updateTimeInView(editTextField: EditText) {
        val myFormat = "HH:mm"
        val sdf = SimpleDateFormat(myFormat, Locale.US)
        editTextField.setText(sdf.format(cal.getTime()))
        editTextField.setError(null)
    }

    private fun setValidators() {
        awesomeValidation.addValidation(this, day_match.id, SimpleCustomValidation { !day_match.text.isNullOrBlank() }, R.string.error_validation_required)
        awesomeValidation.addValidation(this, start_time.id, SimpleCustomValidation { !start_time.text.isNullOrBlank() }, R.string.error_validation_required)
        awesomeValidation.addValidation(this, field_match.id, SimpleCustomValidation { !field_match.text.isNullOrBlank() }, R.string.error_validation_required)
        awesomeValidation.addValidation(this, rival_match.id, SimpleCustomValidation { !rival_match.text.isNullOrBlank() }, R.string.error_validation_required)
    }
}