package com.jesuscb.footcoach.view.match

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.support.v7.app.AlertDialog
import android.util.Log
import android.view.MenuItem
import android.widget.Toast
import com.jesuscb.footcoach.R
import com.jesuscb.footcoach.model.Match
import com.jesuscb.footcoach.model.Player
import com.jesuscb.footcoach.presenter.match.MatchDetailPresenter
import com.jesuscb.footcoach.view.core.BaseActivity
import com.jesuscb.footcoach.view.core.IPlayerListToActivity
import com.jesuscb.footcoach.view.player.PlayerAddToActivity
import com.jesuscb.footcoach.view.player.PlayerListFragment
import kotlinx.android.synthetic.main.activity_detail_match.*

class MatchDetailActivity : BaseActivity(), MatchDetailView, IPlayerListToActivity {


    private val TAG = "MatchDetailActivity"

    private val TAG_ADD_PLAYER_TO_MATCH_RESULT = 1
    private val TAG_EDIT_MATCH_RESULT = 2

    private lateinit var matchDetailPresenter: MatchDetailPresenter

    private var playerListFragment: PlayerListFragment = PlayerListFragment.newInstance(1)

    private var matchId: String? = null
    private var match: Match? = null

    companion object {
        private val ARG_PARAM_MATCH_ID = "matchID"

        fun newIntent(context: Context, id: String): Intent {
            val intent = Intent(context, MatchDetailActivity::class.java)
            intent.putExtra(ARG_PARAM_MATCH_ID, id)
            return intent
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_detail_match)
        setSupportActionBar(details_match_toolbar)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        matchId = intent.getStringExtra(MatchDetailActivity.ARG_PARAM_MATCH_ID)

        matchDetailPresenter = MatchDetailPresenter(this)
        matchDetailPresenter.start()
    }

    override fun initialise() {
        edit_match_btn.setOnClickListener { goToEditMatchActivity() }
        delete_match_btn.setOnClickListener { createConfirmationDialog() }
        addFragment(playerListFragment, fragment_container.id)
        if (!matchId.isNullOrBlank()) {
            matchDetailPresenter.getMatch(matchId!!)
        }
    }

    private fun goToEditMatchActivity() {
        val intent = MatchCRUDActivity.newIntent(this, matchId!!)
        startActivityForResult(intent, TAG_EDIT_MATCH_RESULT)
    }

    override fun updatePlayerList(players: ArrayList<String>) {
        playerListFragment.refreshList(players)
    }


    override fun deletePlayer(player: String) {
        matchDetailPresenter.deletePlayerFromMatch(player, matchId!!)
        matchDetailPresenter.getMatch(matchId!!)
    }


    override fun addPlayersToActivity(players: ArrayList<String>) {
        val intent = PlayerAddToActivity.newIntent(this, players)
        startActivityForResult(intent, TAG_ADD_PLAYER_TO_MATCH_RESULT)
    }

    override fun updateDetailsMatch(match: Match) {
        this.match = match
        date.setText(match.date)
        start_time.setText(match.start_time)
        end_time.setText(match.end_time)
        field.setText(match.field)
        rival.setText(match.rival)
    }

    override fun returnMatchList() {
        val returnIntent = Intent()
        setResult(Activity.RESULT_CANCELED, returnIntent)
        finish()
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.getItemId()) {
            android.R.id.home -> returnMatchList()
        }
        return true
    }

    override fun matchDeleted() {
        val returnIntent = Intent()
        setResult(Activity.RESULT_OK, returnIntent)
        finish()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        when (requestCode) {
            TAG_ADD_PLAYER_TO_MATCH_RESULT -> {
                when (resultCode) {
                    Activity.RESULT_OK -> {
                        val selectedPlayers: ArrayList<Player> = data!!.getSerializableExtra(PlayerAddToActivity.RETURN_PARAM_PLAYERS_SELECTED) as ArrayList<Player>
                        matchDetailPresenter.addSelectedPlayers(selectedPlayers, matchId!!)
                    }
                    Activity.RESULT_CANCELED -> {
                        Log.i(TAG, "Operación cancelada")
                    }
                }
            }
            TAG_EDIT_MATCH_RESULT -> {
                when (resultCode) {
                    Activity.RESULT_OK -> {
                        showToastSuccessModifiedMatch()
                    }
                    Activity.RESULT_CANCELED -> {
                        Log.i(TAG, "Operación cancelada")
                    }
                }
            }
        }
    }

    override fun showToastSuccessAddPlayersToMatchAndRefresh() {
        Toast.makeText(this, getString(R.string.success_adding_players), Toast.LENGTH_SHORT).show()
        matchDetailPresenter.getMatch(matchId!!)
    }

    override fun showToastErrorAddPlayersToMatch() {
        Toast.makeText(this, getString(R.string.error_adding_players), Toast.LENGTH_SHORT).show()
        matchDetailPresenter.getMatch(matchId!!)
    }

    private fun showToastSuccessModifiedMatch() {
        Toast.makeText(this, getString(R.string.success_modified_match), Toast.LENGTH_SHORT).show()
        matchDetailPresenter.getMatch(matchId!!)
    }

    private fun createConfirmationDialog() {
        val mBuilder = AlertDialog.Builder(this)
        mBuilder.setTitle(getString(R.string.text_delete_match))

        mBuilder.setMessage(getString(R.string.text_delete_match_confirm_message))

        mBuilder.setPositiveButton(getString(R.string.text_yes)) { _, _ ->
            matchDetailPresenter.deleteMatch(match!!)
        }
        mBuilder.setNeutralButton(getString(R.string.text_cancel)) { dialog, _ ->
            dialog.cancel()
        }

        val mDialog = mBuilder.create()
        mDialog.show()
    }

    override fun informPlayersIgnored(players: ArrayList<Player>) {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

}
