package com.jesuscb.footcoach.view.match

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.support.design.widget.FloatingActionButton
import android.support.v4.app.Fragment
import android.support.v4.widget.SwipeRefreshLayout
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import android.widget.ProgressBar
import android.widget.Toast
import com.jesuscb.footcoach.R
import com.jesuscb.footcoach.model.Match
import com.jesuscb.footcoach.presenter.match.MatchListPresenter
import com.jesuscb.footcoach.view.core.Utils
import kotlinx.android.synthetic.main.fragment_list_matchs.*

class MatchListFragment : Fragment(), MatchListView {

    private lateinit var matchAdapter: MatchAdapter
    private lateinit var matchListPresenter: MatchListPresenter
    private lateinit var swipeRefreshLayout: SwipeRefreshLayout
    private lateinit var progressBar: ProgressBar

    private val TAG_ADD_MATCH_RESULT = 1
    private val TAG_DETAILS_MATCH_RESULT = 2

    private val TAG = "MatchListFragment"

    @Override
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState);
        matchListPresenter = MatchListPresenter(this)
        matchListPresenter.start()
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        // Inflate the layout for this fragment
        var view: View = inflater.inflate(R.layout.fragment_list_matchs, container, false)

        //RecyclerView
        val rv = view.findViewById<RecyclerView>(R.id.list_matchs)
        rv.layoutManager = LinearLayoutManager(context, LinearLayout.VERTICAL, false)
        rv.adapter = matchAdapter

        //Floating Action Button
        val fab: FloatingActionButton = view.findViewById<FloatingActionButton>(R.id.fab_add_match)
        fab.setOnClickListener { newMatch() }

        //Swipe Refresh Layout
        swipeRefreshLayout = view.findViewById<SwipeRefreshLayout>(R.id.swipe_list_matchs);
        progressBar = view.findViewById<ProgressBar>(R.id.progressBar_matchs);
        initSwipeToRefreshView()

        progressBar.visibility = View.VISIBLE
        //Get match list
        matchListPresenter.getMatchs()
        return view
    }

    override fun initialise() {
        matchAdapter = MatchAdapter(mutableListOf(), this)
    }

    private fun newMatch() {
        val intent = MatchCRUDActivity.newIntent(context!!, "")
        startActivityForResult(intent, TAG_ADD_MATCH_RESULT)
    }

    fun viewDetailsMatch(id: String) {
        val intent = MatchDetailActivity.newIntent(context!!, id)
        startActivityForResult(intent, TAG_DETAILS_MATCH_RESULT)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        when (requestCode) {
            TAG_ADD_MATCH_RESULT -> {
                when (resultCode) {
                    Activity.RESULT_OK -> {
                        Toast.makeText(context, getString(R.string.success_adding_match), Toast.LENGTH_SHORT).show()
                        matchListPresenter.getMatchs()
                    }
                    Activity.RESULT_CANCELED -> {
                        Log.i(TAG, "Operación cancelada")
                    }
                    else -> {
                        showToastErrorAddMatch()
                    }
                }
            }
            TAG_DETAILS_MATCH_RESULT -> {
                when (resultCode) {
                    Activity.RESULT_OK -> {
                        Toast.makeText(context, getString(R.string.success_deleting_match), Toast.LENGTH_SHORT).show()
                        matchListPresenter.getMatchs()
                    }
                    Activity.RESULT_CANCELED -> {
                        Log.i(TAG, "Operación cancelada")
                    }
                    else -> {
                        showToastErrorAddMatch()
                    }
                }
            }
        }
    }

    private fun showToastErrorAddMatch() {
        Toast.makeText(context, getString(R.string.error_adding_match), Toast.LENGTH_SHORT).show()
    }

    override fun updateList(matchList: MutableList<Match>) {
        val matchListOrdered = matchList.sortedWith(Utils.CompareMatchsDatesDescending).toMutableList()
        this.matchAdapter.setItems(matchListOrdered)
        this.matchAdapter.notifyDataSetChanged()
        if (matchList.isEmpty()) {
            emptyState.visibility = View.VISIBLE
        } else {
            emptyState.visibility = View.GONE
        }
    }

    // Initialize my Swipe-to-refresh
    private fun initSwipeToRefreshView() {
        swipeRefreshLayout.setEnabled(true)
        swipeRefreshLayout.setOnRefreshListener(
                SwipeRefreshLayout.OnRefreshListener {
                    matchListPresenter.getMatchs()
                }
        )
    }

    override fun showRefreshProgressBar() {
        if (!swipeRefreshLayout.isRefreshing()) {
            swipeRefreshLayout.post(Runnable {
                swipeRefreshLayout.setRefreshing(true)
                swipeRefreshLayout.setEnabled(false)
            })
        }
    }

    override fun hideRefreshProgressBar() {
        if (swipeRefreshLayout.isRefreshing()) {
            swipeRefreshLayout.post(Runnable {
                swipeRefreshLayout.setEnabled(true)
                swipeRefreshLayout.setRefreshing(false)
            })
        }
        progressBar.visibility = View.GONE
    }

}