package com.jesuscb.footcoach.view.player

import android.app.Activity
import android.app.DatePickerDialog
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.support.v7.app.AlertDialog
import android.support.v7.app.AppCompatActivity
import android.view.MenuItem
import android.widget.DatePicker
import com.basgeekball.awesomevalidation.AwesomeValidation
import com.basgeekball.awesomevalidation.ValidationStyle
import com.basgeekball.awesomevalidation.utility.custom.SimpleCustomValidation
import com.google.common.collect.Range
import com.jesuscb.footcoach.R
import com.jesuscb.footcoach.model.Match
import com.jesuscb.footcoach.model.Player
import com.jesuscb.footcoach.model.Training
import com.jesuscb.footcoach.presenter.player.PlayerCRUDPresenter
import kotlinx.android.synthetic.main.activity_crud_player.*
import java.text.SimpleDateFormat
import java.util.*


class PlayerCRUDActivity : AppCompatActivity(), PlayerCRUDView {

    private val TAG = "AddPlayerActivity"
    private val DEMARCATION = "demarcation"
    private val POSITION = "position"

    private lateinit var playerCRUDPresenter: PlayerCRUDPresenter
    private var cal: Calendar = Calendar.getInstance()
    private var dateSetListener: DatePickerDialog.OnDateSetListener? = null
    private lateinit var awesomeValidation: AwesomeValidation

    private var demarcationIndex: Int? = null
    private var positionIndex: Int? = null

    private lateinit var playerId: String

    companion object {
        private val ARG_PARAM_PLAYER_ID = "playerID"

        fun newIntent(context: Context, id: String): Intent {
            val intent = Intent(context, PlayerCRUDActivity::class.java)
            intent.putExtra(ARG_PARAM_PLAYER_ID, id)
            return intent
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_crud_player)
        setSupportActionBar(add_player_toolbar)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        playerId = intent.getStringExtra(PlayerCRUDActivity.ARG_PARAM_PLAYER_ID)

        playerCRUDPresenter = PlayerCRUDPresenter(this)
        playerCRUDPresenter.start()
    }

    override fun initialise() {
        awesomeValidation = AwesomeValidation(ValidationStyle.BASIC);
        createDateSetListener()
        setValidators()
        birthdate.setOnClickListener { showDatePickerDialog() }
        demarcation.setOnClickListener { createSingleSelectDialog(DEMARCATION, resources.getStringArray(R.array.demarcation_values)) }
        position.setOnClickListener { createSingleSelectDialog(POSITION, resources.getStringArray(R.array.position_goalkeep_values)) }
        add_player_button.setOnClickListener { validatePlayer() }
        if (!playerId.isNullOrBlank()) {
            supportActionBar!!.title = getString(R.string.text_details_player)
            playerCRUDPresenter.getPlayer(playerId)
        }
    }

    override fun playerModified() {
        val returnIntent = Intent()
        setResult(Activity.RESULT_OK, returnIntent)
        finish()
    }

    override fun playerNotModified() {
        val returnIntent = Intent()
        setResult(Activity.RESULT_CANCELED, returnIntent)
        finish()
    }

    override fun updateDetailsPlayer(player: Player) {
        name.setText(player.name)
        identity.setText(player.identity)
        surname.setText(player.surname)
        birthdate.setText(player.birthdate)
        dorsal.setText(player.numDorsal.toString())
        demarcationIndex = resources.getStringArray(R.array.demarcation_keys).indexOf(player.demarcation)
        if (demarcationIndex != null && demarcationIndex != -1) {
            demarcation.setText(resources.getStringArray(R.array.demarcation_values)[demarcationIndex!!])
            when (demarcationIndex) {
                0 -> {
                    positionIndex = resources.getStringArray(R.array.position_goalkeep_keys).indexOf(player.position)
                    if (positionIndex != null && positionIndex != -1) {
                        position.setText(resources.getStringArray(R.array.position_goalkeep_values)[positionIndex!!])
                        position.setOnClickListener { createSingleSelectDialog(POSITION, resources.getStringArray(R.array.position_goalkeep_values)) }
                    }
                }
                1 -> {
                    positionIndex = resources.getStringArray(R.array.position_defenses_keys).indexOf(player.position)
                    if (positionIndex != null && positionIndex != -1) {
                        position.setText(resources.getStringArray(R.array.position_defenses_values)[positionIndex!!])
                        position.setOnClickListener { createSingleSelectDialog(POSITION, resources.getStringArray(R.array.position_defenses_values)) }
                    }
                }
                2 -> {
                    positionIndex = resources.getStringArray(R.array.position_medium_keys).indexOf(player.position)
                    if (positionIndex != null && positionIndex != -1) {
                        position.setText(resources.getStringArray(R.array.position_medium_values)[positionIndex!!])
                        position.setOnClickListener { createSingleSelectDialog(POSITION, resources.getStringArray(R.array.position_medium_values)) }
                    }
                }
                3 -> {
                    positionIndex = resources.getStringArray(R.array.position_attack_keys).indexOf(player.position)
                    if (positionIndex != null && positionIndex != -1) {
                        position.setText(resources.getStringArray(R.array.position_attack_values)[positionIndex!!])
                        position.setOnClickListener { createSingleSelectDialog(POSITION, resources.getStringArray(R.array.position_attack_values)) }
                    }
                }
            }
        }
    }

    override fun hideProgressBar() {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun showProgressBar() {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.getItemId()) {
            android.R.id.home -> playerNotModified()
        }
        return true
    }

    private fun validatePlayer() {
        if (awesomeValidation.validate()) {
            var player: Player = Player(identity.text.toString(), name.text.toString(), surname.text.toString(), birthdate.text.toString(), "", resources.getStringArray(R.array.demarcation_keys)[demarcationIndex!!], dorsal.text.toString().toInt())
            when (demarcationIndex) {
                0 -> {
                    player.position = resources.getStringArray(R.array.position_goalkeep_keys)[positionIndex!!]
                }
                1 -> {
                    player.position = resources.getStringArray(R.array.position_defenses_keys)[positionIndex!!]
                }
                2 -> {
                    player.position = resources.getStringArray(R.array.position_medium_keys)[positionIndex!!]
                }
                3 -> {
                    player.position = resources.getStringArray(R.array.position_attack_keys)[positionIndex!!]
                }
            }
            if (!playerId.isNullOrBlank()) {
                player.id_player = playerId
            }
            playerCRUDPresenter.addOrUpdatePlayer(player)
        }
    }

    private fun setValidators() {
        awesomeValidation.addValidation(this, name.id, SimpleCustomValidation { !name.text.isNullOrBlank() }, R.string.error_validation_required)
        awesomeValidation.addValidation(this, surname.id, SimpleCustomValidation { !surname.text.isNullOrBlank() }, R.string.error_validation_required)
        awesomeValidation.addValidation(this, identity.id, SimpleCustomValidation { !identity.text.isNullOrBlank() }, R.string.error_validation_required)
        awesomeValidation.addValidation(this, dorsal.id, Range.closed(1, 100), R.string.error_validation_dorsal)
        awesomeValidation.addValidation(this, demarcation.id, SimpleCustomValidation { demarcationIndex != null }, R.string.error_validation_required)
        awesomeValidation.addValidation(this, position.id, SimpleCustomValidation { positionIndex != null }, R.string.error_validation_required)
        awesomeValidation.addValidation(this, birthdate.id, SimpleCustomValidation { !birthdate.text.isNullOrBlank() }, R.string.error_validation_required)
    }

    private fun showDatePickerDialog() {
        DatePickerDialog(this@PlayerCRUDActivity,
                dateSetListener,
                cal.get(Calendar.YEAR),
                cal.get(Calendar.MONTH),
                cal.get(Calendar.DAY_OF_MONTH)).show()
    }

    private fun createDateSetListener() {
        dateSetListener = object : DatePickerDialog.OnDateSetListener {
            override fun onDateSet(view: DatePicker, year: Int, monthOfYear: Int,
                                   dayOfMonth: Int) {
                cal.set(Calendar.YEAR, year)
                cal.set(Calendar.MONTH, monthOfYear)
                cal.set(Calendar.DAY_OF_MONTH, dayOfMonth)
                updateDateInView()
            }
        }
    }

    private fun updateDateInView() {
        val myFormat = "dd/MM/yyyy"
        val sdf = SimpleDateFormat(myFormat, Locale.US)
        birthdate.setText(sdf.format(cal.getTime()))
        birthdate.setError(null)
    }

    private fun createSingleSelectDialog(field: String, items: Array<String>) {
        val mBuilder = AlertDialog.Builder(this)
        mBuilder.setTitle(getString(R.string.text_choose_item))
        if (field.equals(DEMARCATION)) {
            mBuilder.setSingleChoiceItems(items, -1) { dialogInterface, i ->
                demarcation.setText(items[i])
                demarcation.setError(null)
                demarcationIndex = i
                when (i) {
                    0 -> {
                        position.setOnClickListener { createSingleSelectDialog(POSITION, resources.getStringArray(R.array.position_goalkeep_values)) }
                    }
                    1 -> {
                        position.setOnClickListener { createSingleSelectDialog(POSITION, resources.getStringArray(R.array.position_defenses_values)) }
                    }
                    2 -> {
                        position.setOnClickListener { createSingleSelectDialog(POSITION, resources.getStringArray(R.array.position_medium_values)) }
                    }
                    3 -> {
                        position.setOnClickListener { createSingleSelectDialog(POSITION, resources.getStringArray(R.array.position_attack_values)) }
                    }
                }
                position.setText("")
                position.setError(resources.getText(R.string.error_validation_required))
                positionIndex = null
                dialogInterface.dismiss()
            }
        } else if (field.equals(POSITION)) {
            mBuilder.setSingleChoiceItems(items, -1) { dialogInterface, i ->
                position.setText(items[i])
                position.setError(null)
                positionIndex = i
                dialogInterface.dismiss()
            }
        }
        mBuilder.setNeutralButton(getString(R.string.text_cancel)) { dialog, _ ->
            dialog.cancel()
        }
        val mDialog = mBuilder.create()
        mDialog.show()
    }

    override fun updateDetailsPlayerTrainings(trainings: ArrayList<Training>) {
        //TODO
    }

    override fun updateDetailsPlayerMatchs(matchs: ArrayList<Match>) {
        //TODO
    }
}
