package com.jesuscb.footcoach.view.player

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.util.Log
import android.view.MenuItem
import android.widget.Toast
import com.jesuscb.footcoach.R
import com.jesuscb.footcoach.model.Match
import com.jesuscb.footcoach.model.Player
import com.jesuscb.footcoach.model.Training
import com.jesuscb.footcoach.presenter.player.PlayerCRUDPresenter
import kotlinx.android.synthetic.main.activity_player_detail.*

class PlayerDetailActivity : AppCompatActivity(), PlayerCRUDView {

    private val TAG = "PlayerDetailActivity"

    private val TAG_EDIT_PLAYER_RESULT = 1

    private lateinit var playerCRUDPresenter: PlayerCRUDPresenter

    private var playerId: String? = null
    private var player: Player? = null

    companion object {
        private val ARG_PARAM_PLAYER_ID = "playerID"

        fun newIntent(context: Context, id: String): Intent {
            val intent = Intent(context, PlayerDetailActivity::class.java)
            intent.putExtra(ARG_PARAM_PLAYER_ID, id)
            return intent
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_player_detail)
        setSupportActionBar(details_player_toolbar)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        playerId = intent.getStringExtra(PlayerDetailActivity.ARG_PARAM_PLAYER_ID)

        playerCRUDPresenter = PlayerCRUDPresenter(this)
        playerCRUDPresenter.start()
    }

    override fun initialise() {
        edit_player_btn.setOnClickListener { goToEditPlayerActivity() }
        playerCRUDPresenter.getPlayer(playerId!!)

    }

    private fun goToEditPlayerActivity() {
        val intent = PlayerCRUDActivity.newIntent(this, playerId!!)
        startActivityForResult(intent, TAG_EDIT_PLAYER_RESULT)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.getItemId()) {
            android.R.id.home -> playerNotModified()
        }
        return true
    }

    override fun playerModified() {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun playerNotModified() {
        val returnIntent = Intent()
        setResult(Activity.RESULT_CANCELED, returnIntent)
        finish()
    }

    override fun updateDetailsPlayer(player: Player) {
        this.player = player
        complete_name.setText(player.name.plus(" ").plus(player.surname))
        birthdate.setText(player.birthdate)
        num_dorsal.setText(player.numDorsal.toString())
        val demarcationIndex = resources.getStringArray(R.array.demarcation_keys).indexOf(player.demarcation)
        var pos: String = ""
        if (demarcationIndex != -1) {
            pos = resources.getStringArray(R.array.demarcation_values)[demarcationIndex]
            when (demarcationIndex) {
                0 -> {
                    val positionIndex = resources.getStringArray(R.array.position_goalkeep_keys).indexOf(player.position)
                    if (positionIndex != -1) {
                        pos = pos.plus(" - ").plus(resources.getStringArray(R.array.position_goalkeep_values)[positionIndex])
                    }
                }
                1 -> {
                    val positionIndex = resources.getStringArray(R.array.position_defenses_keys).indexOf(player.position)
                    if (positionIndex != -1) {
                        pos = pos.plus(" - ").plus(resources.getStringArray(R.array.position_defenses_values)[positionIndex])
                    }
                }
                2 -> {
                    val positionIndex = resources.getStringArray(R.array.position_medium_keys).indexOf(player.position)
                    if (positionIndex != -1) {
                        pos = pos.plus(" - ").plus(resources.getStringArray(R.array.position_medium_values)[positionIndex])
                    }
                }
                3 -> {
                    val positionIndex = resources.getStringArray(R.array.position_attack_keys).indexOf(player.position)
                    if (positionIndex != -1) {
                        pos = pos.plus(" - ").plus(resources.getStringArray(R.array.position_attack_values)[positionIndex])
                    }
                }
            }
        }
        position.setText(pos)
    }

    override fun updateDetailsPlayerTrainings(trainings: ArrayList<Training>) {
        num_trainings.setText(player!!.ids_trainings.size.toString() + " / " + trainings.size.toString())
    }

    override fun updateDetailsPlayerMatchs(matchs: ArrayList<Match>) {
        num_matchs.setText(player!!.ids_matchs.size.toString() + " / " + matchs.size.toString())
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        when (requestCode) {
            TAG_EDIT_PLAYER_RESULT -> {
                when (resultCode) {
                    Activity.RESULT_OK -> {
                        Toast.makeText(this, getString(R.string.success_modified_player), Toast.LENGTH_SHORT).show()
                        playerCRUDPresenter.getPlayer(playerId!!)
                    }
                    Activity.RESULT_CANCELED -> {
                        Log.i(TAG, "Operación cancelada")
                    }
                    else -> {
                        showToastErrorUpdatePlayer()
                    }
                }
            }
        }
    }

    private fun showToastErrorUpdatePlayer() {
        Toast.makeText(this, getString(R.string.error_updating_player), Toast.LENGTH_SHORT).show()
    }

    override fun hideProgressBar() {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun showProgressBar() {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }
}
