package com.jesuscb.footcoach.view.player

import android.app.Activity.RESULT_CANCELED
import android.app.Activity.RESULT_OK
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.support.design.widget.FloatingActionButton
import android.support.v4.app.Fragment
import android.support.v4.widget.SwipeRefreshLayout
import android.support.v7.app.AlertDialog
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.LinearLayout
import android.widget.ProgressBar
import android.widget.Toast
import com.jesuscb.footcoach.R
import com.jesuscb.footcoach.model.Player
import com.jesuscb.footcoach.presenter.player.PlayerListPresenter
import com.jesuscb.footcoach.view.core.IPlayerListToActivity
import kotlinx.android.synthetic.main.fragment_list_players.*

class PlayerListFragment : Fragment(), PlayerListView {

    private lateinit var playersAdapter: PlayerAdapter
    private lateinit var playerDetailListAdapter: PlayerDetailListAdapter
    private lateinit var playersAddToAdapter: PlayerAddToAdapter
    private lateinit var playerListPresenter: PlayerListPresenter
    private lateinit var swipeRefreshLayout: SwipeRefreshLayout
    private lateinit var progressBar: ProgressBar
    private var method: Int? = null

    private var activityCallback: IPlayerListToActivity? = null

    private val TAG_ADD_PLAYER_RESULT = 1
    private val TAG_DETAILS_PLAYER_RESULT = 2
    private val TAG = "PlayerListFragment"

    companion object {
        private val ARG_FRAGMENT_METHOD_ID = "methodID"
        private val ARG_FRAGMENT_PLAYERS_ID = "playersID"

        fun newInstance(method: Int): PlayerListFragment {
            val args: Bundle = Bundle()
            args.putInt(ARG_FRAGMENT_METHOD_ID, method)
            val fragment = PlayerListFragment()
            fragment.arguments = args
            return fragment
        }

        fun newInstanceSelectedPlayers(method: Int, selectPlayers: ArrayList<String>): PlayerListFragment {
            val args: Bundle = Bundle()
            args.putInt(PlayerListFragment.ARG_FRAGMENT_METHOD_ID, method)
            args.putStringArrayList(PlayerListFragment.ARG_FRAGMENT_PLAYERS_ID, selectPlayers)
            val fragment = PlayerListFragment()
            fragment.arguments = args
            return fragment
        }
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        try {
            activityCallback = context as IPlayerListToActivity
        } catch (e: ClassCastException) {
            throw ClassCastException(context.toString() + " must implement IPlayerListToActivity")
        }
    }

    override fun onDetach() {
        activityCallback = null
        super.onDetach()
    }

    @Override
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState);
        playerListPresenter = PlayerListPresenter(this)
        if (arguments != null && arguments!!.containsKey(PlayerListFragment.ARG_FRAGMENT_METHOD_ID)) {
            method = arguments!!.getInt(PlayerListFragment.ARG_FRAGMENT_METHOD_ID)
            when (method) {
                1 -> {
                    playerListPresenter.startPlayerListTrainingDetail()
                }
                2 -> {
                    var playersToIgnore = arguments!!.getStringArrayList(PlayerListFragment.ARG_FRAGMENT_PLAYERS_ID)
                    playerListPresenter.startPlayerAddToTrainingAdapter(playersToIgnore)
                }
            }
        } else {
            playerListPresenter.start()
        }
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        when (method) {
            1 -> {
                val view: View = inflater.inflate(R.layout.fragment_list_players_detail_list, container, false)

                val rv = view.findViewById<RecyclerView>(R.id.list_players)
                rv.layoutManager = LinearLayoutManager(context, LinearLayout.VERTICAL, false)
                rv.adapter = playerDetailListAdapter

                val btn: Button = view.findViewById<Button>(R.id.add_players_btn)
                btn.setOnClickListener { addPlayersToTraining() }

                playerListPresenter.getPlayersFromIds(arrayListOf())
                return view
            }
            2 -> {
                val view: View = inflater.inflate(R.layout.fragment_list_players_add_to_training, container, false)

                val rv = view.findViewById<RecyclerView>(R.id.list_players)
                rv.layoutManager = LinearLayoutManager(context, LinearLayout.VERTICAL, false)
                rv.adapter = playersAddToAdapter

                playerListPresenter.getPlayersRestricted()
                return view
            }
            else -> {
                val view: View = inflater.inflate(R.layout.fragment_list_players, container, false)

                val rv = view.findViewById<RecyclerView>(R.id.list_players)
                rv.layoutManager = LinearLayoutManager(context, LinearLayout.VERTICAL, false)
                rv.adapter = playersAdapter

                val fab: FloatingActionButton = view.findViewById<FloatingActionButton>(R.id.fab_add_player)
                fab.setOnClickListener { newPlayer() }

                swipeRefreshLayout = view.findViewById<SwipeRefreshLayout>(R.id.swipe_list_players);
                progressBar = view.findViewById<ProgressBar>(R.id.progressBar_players);
                initSwipeToRefreshView()

                progressBar.visibility = View.VISIBLE
                playerListPresenter.getPlayers()
                return view
            }
        }
    }

    override fun initialise() {
        playersAdapter = PlayerAdapter(mutableListOf(), this)
    }

    override fun initialisePlayerListTrainingDetail() {
        playerDetailListAdapter = PlayerDetailListAdapter(mutableListOf(), this)
    }

    override fun initialisePlayerListAddToTraining() {
        playersAddToAdapter = PlayerAddToAdapter(mutableListOf(), this)
    }

    override fun updateList(playerList: MutableList<Player>) {
        when (method) {
            2 -> {
                playersAddToAdapter.setItems(playerList)
                playersAddToAdapter.notifyDataSetChanged()
            }
            else -> {
                playersAdapter.setItems(playerList)
                playersAdapter.notifyDataSetChanged()
                if (playerList.isEmpty()) {
                    emptyState.visibility = View.VISIBLE
                } else {
                    emptyState.visibility = View.GONE
                }
            }
        }
    }

    override fun informPlayersIgnored(playersToIgnore: ArrayList<Player>) {
       activityCallback!!.informPlayersIgnored(playersToIgnore)
    }

    override fun addItemToList(player: Player) {
        playerDetailListAdapter.addItem(player)
        playerDetailListAdapter.notifyDataSetChanged()
    }

    fun refreshList(players: ArrayList<String>) {
        playerDetailListAdapter.playerList.clear()
        if (players.isEmpty()) {
            playerDetailListAdapter.notifyDataSetChanged()
        } else {
            playerListPresenter.getPlayersFromIds(players)
        }
    }

    fun getPlayersFromAdapter(): ArrayList<Player> {
        return playersAddToAdapter.getSelectedPlayers()
    }

    fun deletePlayerFromTraining(idPlayer: String) {
        activityCallback!!.deletePlayer(idPlayer)
    }

    private fun addPlayersToTraining() {
        activityCallback!!.addPlayersToActivity(playerDetailListAdapter.playersIds())
    }

    private fun newPlayer() {
        val intent = PlayerCRUDActivity.newIntent(context!!, "")
        startActivityForResult(intent, TAG_ADD_PLAYER_RESULT)
    }

    fun viewDetailsPlayer(id: String) {
        val intent = PlayerDetailActivity.newIntent(context!!, id)
        startActivityForResult(intent, TAG_DETAILS_PLAYER_RESULT)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        when (requestCode) {
            TAG_ADD_PLAYER_RESULT -> {
                when (resultCode) {
                    RESULT_OK -> {
                        Toast.makeText(context, getString(R.string.success_adding_player), Toast.LENGTH_SHORT).show()
                        playerListPresenter.getPlayers()
                    }
                    RESULT_CANCELED -> {
                        Log.i(TAG, "Operación cancelada")
                    }
                    else -> {
                        showToastErrorAddPlayer()
                    }
                }
            }
            TAG_DETAILS_PLAYER_RESULT -> {
                when (resultCode) {
                    RESULT_OK -> {
                        Toast.makeText(context, getString(R.string.success_modified_player), Toast.LENGTH_SHORT).show()
                        playerListPresenter.getPlayers()
                    }
                    RESULT_CANCELED -> {
                        Log.i(TAG, "Operación cancelada")
                    }
                    else -> {
                        showToastErrorUpdatePlayer()
                    }
                }
            }
        }
    }

    private fun showToastErrorAddPlayer() {
        Toast.makeText(context, getString(R.string.error_adding_player), Toast.LENGTH_SHORT).show()
    }

    private fun showToastErrorUpdatePlayer() {
        Toast.makeText(context, getString(R.string.error_updating_player), Toast.LENGTH_SHORT).show()
    }

    override fun showErrorDeletingPlayer() {
        Toast.makeText(context, getString(R.string.error_deleting_player), Toast.LENGTH_SHORT).show()
    }

    override fun showSuccessDeletingPlayer() {
        Toast.makeText(context, getString(R.string.success_deleting_player), Toast.LENGTH_SHORT).show()
    }

    // Initialize my Swipe-to-refresh
    private fun initSwipeToRefreshView() {
        swipeRefreshLayout.setEnabled(true)
        swipeRefreshLayout.setOnRefreshListener(
                SwipeRefreshLayout.OnRefreshListener {
                    playerListPresenter.getPlayers()
                }
        )
    }

    override fun showRefreshProgressBar() {
        if (!swipeRefreshLayout.isRefreshing()) {
            swipeRefreshLayout.post(Runnable {
                swipeRefreshLayout.setRefreshing(true)
                swipeRefreshLayout.setEnabled(false)
            })
        }
    }

    override fun hideRefreshProgressBar() {
        if (swipeRefreshLayout.isRefreshing()) {
            swipeRefreshLayout.post(Runnable {
                swipeRefreshLayout.setEnabled(true)
                swipeRefreshLayout.setRefreshing(false)
            })
        }
        progressBar.visibility = View.GONE
    }

    fun getDemarcationValueFromKey(key: String): String {
        var value: String = ""
        val demarcationIndex = resources.getStringArray(R.array.demarcation_keys).indexOf(key)
        if (demarcationIndex != -1) {
            value = resources.getStringArray(R.array.demarcation_values)[demarcationIndex]
        }
        return value
    }

    fun deletePlayer(player: Player): Boolean {
        val mBuilder = AlertDialog.Builder(context!!)
        mBuilder.setTitle(getString(R.string.text_delete_player))

        mBuilder.setMessage(getString(R.string.text_delete_player_confirm_message))

        mBuilder.setPositiveButton(getString(R.string.text_yes)) { _, _ ->
            playerListPresenter.deletePlayer(player)
        }
        mBuilder.setNeutralButton(getString(R.string.text_cancel)) { dialog, _ ->
            dialog.cancel()
        }

        val mDialog = mBuilder.create()
        mDialog.show()
        return true
    }
}