package com.jesuscb.footcoach.view.register

import android.content.Intent
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.view.MenuItem
import android.view.View
import android.widget.Toast
import com.basgeekball.awesomevalidation.AwesomeValidation
import com.basgeekball.awesomevalidation.ValidationStyle
import com.basgeekball.awesomevalidation.utility.custom.SimpleCustomValidation
import com.jesuscb.footcoach.R
import com.jesuscb.footcoach.model.User
import com.jesuscb.footcoach.presenter.RegisterPresenter
import com.jesuscb.footcoach.view.login.LoginActivity
import kotlinx.android.synthetic.main.activity_register.*

class RegisterActivity : AppCompatActivity(), RegisterView {

    private val TAG = "RegisterActivity"

    private lateinit var registerPresenter: RegisterPresenter
    private lateinit var awesomeValidation: AwesomeValidation

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_register)
        setSupportActionBar(app_toolbar)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        registerPresenter = RegisterPresenter(this)
        registerPresenter.start()
    }

    override fun initialise() {
        awesomeValidation = AwesomeValidation(ValidationStyle.BASIC);
        setValidators()
        register_button.setOnClickListener { registerUser() }
    }

    private fun setValidators() {
        awesomeValidation.addValidation(this, name.id, SimpleCustomValidation { !name.text.isNullOrBlank() }, R.string.error_validation_required)
        awesomeValidation.addValidation(this, email.id, android.util.Patterns.EMAIL_ADDRESS, R.string.error_validation_email)
        awesomeValidation.addValidation(this, password.id, SimpleCustomValidation { password.text.toString().length >= 6 }, R.string.error_validation_password)
    }

    private fun registerUser() {
        if (awesomeValidation.validate()) {
            var user: User = User(email.text.toString(), password.text.toString())
            user.name = name.text.toString()
            registerPresenter.registerUser(user)
        }
    }

    private fun goToLogin(){
        val intent = Intent(this@RegisterActivity, LoginActivity::class.java)
        intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK)
        startActivity(intent)
    }

    override fun registerSucessfull() {
        goToLogin()
    }

    override fun showRegisterErrorToast() {
        Toast.makeText(this, getString(R.string.error_register), Toast.LENGTH_SHORT).show()
    }

    override fun showSaveUserErrorToast() {
        Toast.makeText(this, getString(R.string.error_save_user_data), Toast.LENGTH_SHORT).show()
    }

    override fun showProgressBar(state: Boolean) {
        if (state) {
            register_progress.visibility = View.VISIBLE
        } else {
            register_progress.visibility = View.GONE
        }
    }

    override fun onBackPressed() {
        goToLogin()
    }

    override fun onOptionsItemSelected(menuItem: MenuItem): Boolean {
        if (menuItem.getItemId() == android.R.id.home) {
            goToLogin()
        }
        return super.onOptionsItemSelected(menuItem)
    }

}