package com.jesuscb.footcoach.view.training

import android.app.Activity
import android.app.DatePickerDialog
import android.app.TimePickerDialog
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.view.MenuItem
import android.widget.DatePicker
import android.widget.EditText
import android.widget.TimePicker
import com.basgeekball.awesomevalidation.AwesomeValidation
import com.basgeekball.awesomevalidation.ValidationStyle
import com.basgeekball.awesomevalidation.utility.custom.SimpleCustomValidation
import com.jesuscb.footcoach.R
import com.jesuscb.footcoach.model.Training
import com.jesuscb.footcoach.presenter.training.TrainingCRUDPresenter
import kotlinx.android.synthetic.main.activity_crud_training.*
import java.text.SimpleDateFormat
import java.util.*

class TrainingCRUDActivity : AppCompatActivity(), TrainingCRUDView {

    private val TAG = "TrainingCRUDActivity"

    private lateinit var trainingCRUDPresenter: TrainingCRUDPresenter
    private lateinit var awesomeValidation: AwesomeValidation
    private var cal: Calendar = Calendar.getInstance()
    private var dateSetListener: DatePickerDialog.OnDateSetListener? = null
    private var timeSetListener: TimePickerDialog.OnTimeSetListener? = null
    private lateinit var trainingId: String
    private var training: Training? = null

    companion object {
        private val ARG_PARAM_TRAINING_ID = "trainingID"

        fun newIntent(context: Context, id: String): Intent {
            val intent = Intent(context, TrainingCRUDActivity::class.java)
            intent.putExtra(ARG_PARAM_TRAINING_ID, id)
            return intent
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_crud_training)
        setSupportActionBar(update_training_toolbar)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        trainingId = intent.getStringExtra(TrainingCRUDActivity.ARG_PARAM_TRAINING_ID)

        trainingCRUDPresenter = TrainingCRUDPresenter(this)
        trainingCRUDPresenter.start()
    }

    override fun initialise() {
        awesomeValidation = AwesomeValidation(ValidationStyle.BASIC);
        createDateSetListener()
        setValidators()
        day_training.setOnClickListener { showDatePickerDialog() }
        start_time.setOnClickListener { showTimePickerDialog(start_time) }
        end_time.setOnClickListener { showTimePickerDialog(end_time) }
        update_training_button.setOnClickListener { validateTraining() }
        if (!trainingId.isNullOrBlank()) {
            supportActionBar!!.title = getString(R.string.text_update_training)
            trainingCRUDPresenter.getTraining(trainingId)
        }
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.getItemId()) {
            android.R.id.home -> trainingNotModified()
        }
        return true
    }

    override fun updateDetailsTraining(training: Training) {
        day_training.setText(training.date)
        start_time.setText(training.start_time)
        end_time.setText(training.end_time)
        this.training = training
    }

    override fun hideProgressBar() {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun showProgressBar() {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun trainingModified() {
        val returnIntent = Intent()
        setResult(Activity.RESULT_OK, returnIntent)
        finish()
    }

    override fun trainingNotModified() {
        val returnIntent = Intent()
        setResult(Activity.RESULT_CANCELED, returnIntent)
        finish()
    }

    private fun validateTraining() {
        if (awesomeValidation.validate()) {
            if (training != null) {
                training!!.start_time = start_time.text.toString()
                training!!.end_time = end_time.text.toString()
                training!!.date = day_training.text.toString()
            } else {
                training = Training(day_training.text.toString(), start_time.text.toString())
                training!!.end_time = end_time.text.toString()
            }
            trainingCRUDPresenter.addOrUpdateTraining(training!!)
        }
    }

    private fun showDatePickerDialog() {
        DatePickerDialog(this@TrainingCRUDActivity,
                dateSetListener,
                cal.get(Calendar.YEAR),
                cal.get(Calendar.MONTH),
                cal.get(Calendar.DAY_OF_MONTH)).show()
    }

    private fun showTimePickerDialog(editTextField: EditText) {
        createTimeSetListener(editTextField)
        TimePickerDialog(this@TrainingCRUDActivity,
                timeSetListener,
                cal.get(Calendar.HOUR_OF_DAY),
                cal.get(Calendar.MINUTE),
                true).show()
    }

    private fun createDateSetListener() {
        dateSetListener = object : DatePickerDialog.OnDateSetListener {
            override fun onDateSet(view: DatePicker, year: Int, monthOfYear: Int,
                                   dayOfMonth: Int) {
                cal.set(Calendar.YEAR, year)
                cal.set(Calendar.MONTH, monthOfYear)
                cal.set(Calendar.DAY_OF_MONTH, dayOfMonth)
                updateDateInView()
            }
        }
    }

    private fun createTimeSetListener(editTextField: EditText) {
        timeSetListener = object : TimePickerDialog.OnTimeSetListener {
            override fun onTimeSet(view: TimePicker?, hourOfDay: Int, minute: Int) {
                cal.set(Calendar.HOUR_OF_DAY, hourOfDay)
                cal.set(Calendar.MINUTE, minute)
                updateTimeInView(editTextField)
            }
        }
    }

    private fun updateDateInView() {
        val myFormat = "dd/MM/yyyy"
        val sdf = SimpleDateFormat(myFormat, Locale.US)
        day_training.setText(sdf.format(cal.getTime()))
        day_training.setError(null)
    }

    private fun updateTimeInView(editTextField: EditText) {
        val myFormat = "HH:mm"
        val sdf = SimpleDateFormat(myFormat, Locale.US)
        editTextField.setText(sdf.format(cal.getTime()))
        editTextField.setError(null)
    }

    private fun setValidators() {
        awesomeValidation.addValidation(this, day_training.id, SimpleCustomValidation { !day_training.text.isNullOrBlank() }, R.string.error_validation_required)
        awesomeValidation.addValidation(this, start_time.id, SimpleCustomValidation { !start_time.text.isNullOrBlank() }, R.string.error_validation_required)

    }
}