package com.jesuscb.footcoach.view.training

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.support.design.widget.TabLayout
import android.support.v4.app.Fragment
import android.support.v4.app.FragmentManager
import android.support.v4.app.FragmentPagerAdapter
import android.support.v7.app.AlertDialog
import android.util.Log
import android.view.MenuItem
import android.widget.Toast
import com.jesuscb.footcoach.R
import com.jesuscb.footcoach.model.Exercise
import com.jesuscb.footcoach.model.Player
import com.jesuscb.footcoach.model.Training
import com.jesuscb.footcoach.presenter.training.TrainingDetailPresenter
import com.jesuscb.footcoach.view.core.BaseActivity
import com.jesuscb.footcoach.view.core.IExerciseListToActivity
import com.jesuscb.footcoach.view.core.IPlayerListToActivity
import com.jesuscb.footcoach.view.exercise.ExerciseAddToTraining
import com.jesuscb.footcoach.view.exercise.ExerciseListFragment
import com.jesuscb.footcoach.view.player.PlayerAddToActivity
import com.jesuscb.footcoach.view.player.PlayerListFragment
import kotlinx.android.synthetic.main.activity_detail_training.*

class TrainingDetailActivity : BaseActivity(), TrainingDetailView, IExerciseListToActivity, IPlayerListToActivity {

    private val TAG = "TrainingDetailActivity"

    private val TAG_ADD_EXERCISE_TO_TRAINING_RESULT = 1
    private val TAG_ADD_PLAYER_TO_TRAINING_RESULT = 2
    private val TAG_EDIT_TRAINING_RESULT = 3

    private lateinit var trainingDetailPresenter: TrainingDetailPresenter

    private var exerciseListFragment: ExerciseListFragment = ExerciseListFragment.newInstance(1)
    private var playerListFragment: PlayerListFragment = PlayerListFragment.newInstance(1)

    private var trainingId: String? = null
    private var training: Training? = null

    private var mSectionsPagerAdapter: SectionsPagerAdapter? = null

    companion object {
        private val ARG_PARAM_TRAINING_ID = "trainingID"

        fun newIntent(context: Context, id: String): Intent {
            val intent = Intent(context, TrainingDetailActivity::class.java)
            intent.putExtra(ARG_PARAM_TRAINING_ID, id)
            return intent
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_detail_training)
        setSupportActionBar(details_training_toolbar)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        mSectionsPagerAdapter = SectionsPagerAdapter(supportFragmentManager)

        // Set up the ViewPager with the sections adapter.
        fragment_container.adapter = mSectionsPagerAdapter

        trainingId = intent.getStringExtra(TrainingDetailActivity.ARG_PARAM_TRAINING_ID)

        trainingDetailPresenter = TrainingDetailPresenter(this)
        trainingDetailPresenter.start()
    }

    override fun initialise() {
        fragment_container.addOnPageChangeListener(TabLayout.TabLayoutOnPageChangeListener(tabs))
        edit_training_btn.setOnClickListener { goToEditTrainingActivity() }
        delete_training_btn.setOnClickListener { createConfirmationDialog() }
        tabs.addOnTabSelectedListener(TabLayout.ViewPagerOnTabSelectedListener(fragment_container))
        if (!trainingId.isNullOrBlank()) {
            trainingDetailPresenter.getTraining(trainingId!!)
        }
    }

    private fun goToEditTrainingActivity() {
        val intent = TrainingCRUDActivity.newIntent(this, trainingId!!)
        startActivityForResult(intent, TAG_EDIT_TRAINING_RESULT)
    }

    override fun updateDataFragments(exercises: ArrayList<String>, players: ArrayList<String>) {
        exerciseListFragment.refreshList(exercises)
        playerListFragment.refreshList(players)
    }

    override fun deleteExercise(exercise: String) {
        trainingDetailPresenter.deleteExerciseFromTraining(exercise, trainingId!!)
        trainingDetailPresenter.getTraining(trainingId!!)
    }

    override fun deletePlayer(player: String) {
        trainingDetailPresenter.deletePlayerFromTraining(player, trainingId!!)
        trainingDetailPresenter.getTraining(trainingId!!)
    }

    override fun addExercisesToTraining(exercisesIds: ArrayList<String>) {
        val intent = ExerciseAddToTraining.newIntent(this, exercisesIds)
        startActivityForResult(intent, TAG_ADD_EXERCISE_TO_TRAINING_RESULT)
    }

    override fun addPlayersToActivity(players: ArrayList<String>) {
        val intent = PlayerAddToActivity.newIntent(this, players)
        startActivityForResult(intent, TAG_ADD_PLAYER_TO_TRAINING_RESULT)
    }

    inner class SectionsPagerAdapter(fm: FragmentManager) : FragmentPagerAdapter(fm) {

        override fun getItem(position: Int): Fragment {
            when (position) {
                0 -> {
                    return exerciseListFragment
                }
                1 -> {
                    return playerListFragment
                }
                else -> {
                    return Fragment()
                }
            }
        }

        override fun getCount(): Int {
            return 2
        }
    }

    override fun updateDetailsTraining(training: Training) {
        this.training=training
        date.setText(training.date)
        start_time.setText(training.start_time)
        end_time.setText(training.end_time)
    }

    override fun returnTrainingList() {
        val returnIntent = Intent()
        setResult(Activity.RESULT_CANCELED, returnIntent)
        finish()
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.getItemId()) {
            android.R.id.home -> returnTrainingList()
        }
        return true
    }

    override fun trainingDeleted() {
        val returnIntent = Intent()
        setResult(Activity.RESULT_OK, returnIntent)
        finish()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        when (requestCode) {
            TAG_ADD_EXERCISE_TO_TRAINING_RESULT -> {
                when (resultCode) {
                    Activity.RESULT_OK -> {
                        val selectedExercises: ArrayList<Exercise> = data!!.getSerializableExtra(ExerciseAddToTraining.RETURN_PARAM_EXERCISES_SELECTED) as ArrayList<Exercise>
                        trainingDetailPresenter.addSelectedExercises(selectedExercises, trainingId!!)
                    }
                    Activity.RESULT_CANCELED -> {
                        Log.i(TAG, "Operación cancelada")
                    }
                }
            }
            TAG_ADD_PLAYER_TO_TRAINING_RESULT -> {
                when (resultCode) {
                    Activity.RESULT_OK -> {
                        val selectedPlayers: ArrayList<Player> = data!!.getSerializableExtra(PlayerAddToActivity.RETURN_PARAM_PLAYERS_SELECTED) as ArrayList<Player>
                        trainingDetailPresenter.addSelectedPlayers(selectedPlayers, trainingId!!)
                    }
                    Activity.RESULT_CANCELED -> {
                        Log.i(TAG, "Operación cancelada")
                    }
                }
            }
            TAG_EDIT_TRAINING_RESULT -> {
                when (resultCode) {
                    Activity.RESULT_OK -> {
                        showToastSuccessModifiedTraining()
                    }
                    Activity.RESULT_CANCELED -> {
                        Log.i(TAG, "Operación cancelada")
                    }
                }
            }
        }
    }

    override fun showToastSuccessAddExercisesToTrainingAndRefresh() {
        Toast.makeText(this, getString(R.string.success_adding_exercises), Toast.LENGTH_SHORT).show()
        trainingDetailPresenter.getTraining(trainingId!!)
    }

    override fun showToastErrorAddExercisesToTraining() {
        Toast.makeText(this, getString(R.string.error_adding_exercises), Toast.LENGTH_SHORT).show()
        trainingDetailPresenter.getTraining(trainingId!!)
    }

    override fun showToastSuccessAddPlayersToTrainingAndRefresh() {
        Toast.makeText(this, getString(R.string.success_adding_players), Toast.LENGTH_SHORT).show()
        trainingDetailPresenter.getTraining(trainingId!!)
    }

    override fun showToastErrorAddPlayersToTraining() {
        Toast.makeText(this, getString(R.string.error_adding_players), Toast.LENGTH_SHORT).show()
        trainingDetailPresenter.getTraining(trainingId!!)
    }

    private fun showToastSuccessModifiedTraining() {
        Toast.makeText(this, getString(R.string.success_modified_training), Toast.LENGTH_SHORT).show()
        trainingDetailPresenter.getTraining(trainingId!!)
    }

    private fun createConfirmationDialog() {
        val mBuilder = AlertDialog.Builder(this)
        mBuilder.setTitle(getString(R.string.text_delete_training))

        mBuilder.setMessage(getString(R.string.text_delete_training_confirm_message))

        mBuilder.setPositiveButton(getString(R.string.text_yes)) { _, _ ->
            trainingDetailPresenter.deleteTraining(training!!)
        }
        mBuilder.setNeutralButton(getString(R.string.text_cancel)) { dialog, _ ->
            dialog.cancel()
        }

        val mDialog = mBuilder.create()
        mDialog.show()
    }

    override fun informPlayersIgnored(players: ArrayList<Player>) {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun informExercisesIgnored(exercisesToIgnore: ArrayList<Exercise>) {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

}
