package com.jesuscb.footcoach.view.training

import android.app.Activity
import android.app.DatePickerDialog
import android.app.TimePickerDialog
import android.content.Intent
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.view.MenuItem
import android.view.View
import android.widget.CompoundButton
import android.widget.DatePicker
import android.widget.EditText
import android.widget.TimePicker
import com.basgeekball.awesomevalidation.AwesomeValidation
import com.basgeekball.awesomevalidation.ValidationStyle
import com.basgeekball.awesomevalidation.utility.custom.SimpleCustomValidation
import com.google.common.collect.Range
import com.jesuscb.footcoach.R
import com.jesuscb.footcoach.model.Training
import com.jesuscb.footcoach.presenter.training.TrainingGeneratorPresenter
import kotlinx.android.synthetic.main.activity_training_generator.*
import java.text.SimpleDateFormat
import java.util.*
import kotlin.collections.HashMap


class TrainingGeneratorActivity : AppCompatActivity(), TrainingGeneratorView {

    private val TAG = "TrainingGeneratorActivity"

    private lateinit var trainingGeneratorPresenter: TrainingGeneratorPresenter
    private lateinit var awesomeValidation: AwesomeValidation
    private var cal: Calendar = Calendar.getInstance()
    private var dateSetListener: DatePickerDialog.OnDateSetListener? = null
    private var timeSetListener: TimePickerDialog.OnTimeSetListener? = null

    private var exercisesTypes: HashMap<String, Int> = HashMap()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_training_generator)
        setSupportActionBar(update_training_toolbar)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        trainingGeneratorPresenter = TrainingGeneratorPresenter(this)
        trainingGeneratorPresenter.start()
    }

    override fun initialise() {
        awesomeValidation = AwesomeValidation(ValidationStyle.BASIC);
        createDateSetListener()
        setValidators()
        day_training.setOnClickListener { showDatePickerDialog() }
        start_time.setOnClickListener { showTimePickerDialog(start_time) }
        check_pretr.setOnCheckedChangeListener(CheckedChangeListener(resources.getString(R.string.PRETR)))
        check_tech.setOnCheckedChangeListener(CheckedChangeListener(resources.getString(R.string.TECH)))
        check_tacdef.setOnCheckedChangeListener(CheckedChangeListener(resources.getString(R.string.TACDEF)))
        check_tacat.setOnCheckedChangeListener(CheckedChangeListener(resources.getString(R.string.TACAT)))
        check_strong.setOnCheckedChangeListener(CheckedChangeListener(resources.getString(R.string.STRONG)))
        check_reacvel.setOnCheckedChangeListener(CheckedChangeListener(resources.getString(R.string.REACVEL)))
        check_expvel.setOnCheckedChangeListener(CheckedChangeListener(resources.getString(R.string.EXPVEL)))
        check_protr.setOnCheckedChangeListener(CheckedChangeListener(resources.getString(R.string.PROTR)))
        generate_training_button.setOnClickListener { validateTraining() }
    }

    override fun trainingGenerated(id: String) {
        val intent = TrainingDetailActivity.newIntent(this, id)
        startActivity(intent)
        finish()
    }


    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.getItemId()) {
            android.R.id.home -> {
                val returnIntent = Intent()
                setResult(Activity.RESULT_CANCELED, returnIntent)
                finish()
            }
        }
        return true
    }

    override fun showProgressBar(state: Boolean) {
        if (state) {
            bar_progress.visibility = View.VISIBLE
        } else {
            bar_progress.visibility = View.GONE
        }
    }

    private fun validateTraining() {
        if (awesomeValidation.validate()) {
            if (validateExercisesDurations()) {
                val training = Training(day_training.text.toString(), start_time.text.toString())
                trainingGeneratorPresenter.generateTraining(training, duration_training.text.toString().toInt(), exercisesTypes)
            }
        }
    }

    private fun showDatePickerDialog() {
        DatePickerDialog(this@TrainingGeneratorActivity,
                dateSetListener,
                cal.get(Calendar.YEAR),
                cal.get(Calendar.MONTH),
                cal.get(Calendar.DAY_OF_MONTH)).show()
    }

    private fun showTimePickerDialog(editTextField: EditText) {
        createTimeSetListener(editTextField)
        TimePickerDialog(this@TrainingGeneratorActivity,
                timeSetListener,
                cal.get(Calendar.HOUR_OF_DAY),
                cal.get(Calendar.MINUTE),
                true).show()
    }

    private fun createDateSetListener() {
        dateSetListener = object : DatePickerDialog.OnDateSetListener {
            override fun onDateSet(view: DatePicker, year: Int, monthOfYear: Int,
                                   dayOfMonth: Int) {
                cal.set(Calendar.YEAR, year)
                cal.set(Calendar.MONTH, monthOfYear)
                cal.set(Calendar.DAY_OF_MONTH, dayOfMonth)
                updateDateInView()
            }
        }
    }

    private fun createTimeSetListener(editTextField: EditText) {
        timeSetListener = object : TimePickerDialog.OnTimeSetListener {
            override fun onTimeSet(view: TimePicker?, hourOfDay: Int, minute: Int) {
                cal.set(Calendar.HOUR_OF_DAY, hourOfDay)
                cal.set(Calendar.MINUTE, minute)
                updateTimeInView(editTextField)
            }
        }
    }

    private fun updateDateInView() {
        val myFormat = "dd/MM/yyyy"
        val sdf = SimpleDateFormat(myFormat, Locale.US)
        day_training.setText(sdf.format(cal.getTime()))
        day_training.setError(null)
    }

    private fun updateTimeInView(editTextField: EditText) {
        val myFormat = "HH:mm"
        val sdf = SimpleDateFormat(myFormat, Locale.US)
        editTextField.setText(sdf.format(cal.getTime()))
        editTextField.setError(null)
    }

    private fun setValidators() {
        awesomeValidation.addValidation(this, day_training.id, SimpleCustomValidation { !day_training.text.isNullOrBlank() }, R.string.error_validation_required)
        awesomeValidation.addValidation(this, start_time.id, SimpleCustomValidation { !start_time.text.isNullOrBlank() }, R.string.error_validation_required)
        awesomeValidation.addValidation(this, duration_training.id, SimpleCustomValidation { !duration_training.text.isNullOrBlank() }, R.string.error_validation_required)
        awesomeValidation.addValidation(this, duration_training.id, Range.closed(1, 180), R.string.error_validation_duration_training_range)
    }

    private fun validateExercisesDurations(): Boolean {
        var validate: Boolean = true
        for ((key, _) in exercisesTypes) {
            when (key) {
                resources.getString(R.string.PRETR_KEY) -> {
                    if (duration_pretr.text.toString().isNullOrBlank()) {
                        validate = false
                        duration_pretr.setError(resources.getText(R.string.error_validation_required))
                    } else {
                        exercisesTypes.put(resources.getString(R.string.PRETR_KEY), duration_pretr.text.toString().toInt())
                    }
                }
                resources.getString(R.string.TECH_KEY) -> {
                    if (duration_tech.text.toString().isNullOrBlank()) {
                        validate = false
                        duration_tech.setError(resources.getText(R.string.error_validation_required))
                    } else {
                        exercisesTypes.put(resources.getString(R.string.TECH_KEY), duration_tech.text.toString().toInt())
                    }
                }
                resources.getString(R.string.TACDEF_KEY) -> {
                    if (duration_tacdef.text.toString().isNullOrBlank()) {
                        validate = false
                        duration_tacdef.setError(resources.getText(R.string.error_validation_required))
                    } else {
                        exercisesTypes.put(resources.getString(R.string.TACDEF_KEY), duration_tacdef.text.toString().toInt())
                    }
                }
                resources.getString(R.string.TACAT_KEY) -> {
                    if (duration_tacat.text.toString().isNullOrBlank()) {
                        validate = false
                        duration_tacat.setError(resources.getText(R.string.error_validation_required))
                    } else {
                        exercisesTypes.put(resources.getString(R.string.TACAT_KEY), duration_tacat.text.toString().toInt())
                    }
                }
                resources.getString(R.string.STRONG_KEY) -> {
                    if (duration_strong.text.toString().isNullOrBlank()) {
                        validate = false
                        duration_strong.setError(resources.getText(R.string.error_validation_required))
                    } else {
                        exercisesTypes.put(resources.getString(R.string.STRONG_KEY), duration_strong.text.toString().toInt())
                    }
                }
                resources.getString(R.string.REACVEL_KEY) -> {
                    if (duration_reacvel.text.toString().isNullOrBlank()) {
                        validate = false
                        duration_reacvel.setError(resources.getText(R.string.error_validation_required))
                    } else {
                        exercisesTypes.put(resources.getString(R.string.REACVEL_KEY), duration_reacvel.text.toString().toInt())
                    }
                }
                resources.getString(R.string.EXPVEL_KEY) -> {
                    if (duration_expvel.text.toString().isNullOrBlank()) {
                        validate = false
                        duration_expvel.setError(resources.getText(R.string.error_validation_required))
                    } else {
                        exercisesTypes.put(resources.getString(R.string.EXPVEL_KEY), duration_expvel.text.toString().toInt())
                    }
                }
                resources.getString(R.string.PROTR_KEY) -> {
                    if (duration_protr.text.toString().isNullOrBlank()) {
                        validate = false
                        duration_protr.setError(resources.getText(R.string.error_validation_required))
                    } else {
                        exercisesTypes.put(resources.getString(R.string.PROTR_KEY), duration_protr.text.toString().toInt())
                    }
                }
            }
        }
        return validate
    }

    inner class CheckedChangeListener(internal var type: String) : CompoundButton.OnCheckedChangeListener {

        override fun onCheckedChanged(buttonView: CompoundButton, isChecked: Boolean) {
            when (type) {
                resources.getString(R.string.PRETR) -> {
                    if (isChecked) {
                        exercisesTypes.put(resources.getString(R.string.PRETR_KEY), 0)
                        duration_pretr.visibility = View.VISIBLE
                        duration_pretr.setText("")
                        duration_pretr.setError(resources.getText(R.string.error_validation_required))
                        duration_pretr.requestFocus()
                    } else {
                        exercisesTypes.remove(resources.getString(R.string.PRETR_KEY))
                        duration_pretr.setText("")
                        duration_pretr.setError(null)
                        duration_pretr.visibility = View.INVISIBLE
                    }
                }
                resources.getString(R.string.TECH) -> {
                    if (isChecked) {
                        duration_tech.visibility = View.VISIBLE
                        exercisesTypes.put(resources.getString(R.string.TECH_KEY), 0)
                        duration_tech.setText("")
                        duration_tech.setError(resources.getText(R.string.error_validation_required))
                        duration_tech.requestFocus()
                    } else {
                        exercisesTypes.remove(resources.getString(R.string.TECH_KEY))
                        duration_tech.setText("")
                        duration_tech.setError(null)
                        duration_tech.visibility = View.INVISIBLE
                    }
                }
                resources.getString(R.string.TACDEF) -> {
                    if (isChecked) {
                        duration_tacdef.visibility = View.VISIBLE
                        exercisesTypes.put(resources.getString(R.string.TACDEF_KEY), 0)
                        duration_tacdef.setText("")
                        duration_tacdef.setError(resources.getText(R.string.error_validation_required))
                        duration_tacdef.requestFocus()
                    } else {
                        exercisesTypes.remove(resources.getString(R.string.TACDEF_KEY))
                        duration_tacdef.setText("")
                        duration_tacdef.setError(null)
                        duration_tacdef.visibility = View.INVISIBLE
                    }
                }
                resources.getString(R.string.TACAT) -> {
                    if (isChecked) {
                        duration_tacat.visibility = View.VISIBLE
                        exercisesTypes.put(resources.getString(R.string.TACAT_KEY), 0)
                        duration_tacat.setText("")
                        duration_tacat.setError(resources.getText(R.string.error_validation_required))
                        duration_tacat.requestFocus()
                    } else {
                        exercisesTypes.remove(resources.getString(R.string.TACAT_KEY))
                        duration_tacat.setText("")
                        duration_tacat.setError(null)
                        duration_tacat.visibility = View.INVISIBLE
                    }
                }
                resources.getString(R.string.STRONG) -> {
                    if (isChecked) {
                        duration_strong.visibility = View.VISIBLE
                        exercisesTypes.put(resources.getString(R.string.STRONG_KEY), 0)
                        duration_strong.setText("")
                        duration_strong.setError(resources.getText(R.string.error_validation_required))
                        duration_strong.requestFocus()
                    } else {
                        exercisesTypes.remove(resources.getString(R.string.STRONG_KEY))
                        duration_strong.setText("")
                        duration_strong.setError(null)
                        duration_strong.visibility = View.INVISIBLE
                    }
                }
                resources.getString(R.string.REACVEL) -> {
                    if (isChecked) {
                        duration_reacvel.visibility = View.VISIBLE
                        exercisesTypes.put(resources.getString(R.string.REACVEL_KEY), 0)
                        duration_reacvel.setText("")
                        duration_reacvel.setError(resources.getText(R.string.error_validation_required))
                        duration_reacvel.requestFocus()
                    } else {
                        exercisesTypes.remove(resources.getString(R.string.REACVEL_KEY))
                        duration_reacvel.setText("")
                        duration_reacvel.setError(null)
                        duration_reacvel.visibility = View.INVISIBLE
                    }
                }
                resources.getString(R.string.EXPVEL) -> {
                    if (isChecked) {
                        duration_expvel.visibility = View.VISIBLE
                        exercisesTypes.put(resources.getString(R.string.EXPVEL_KEY), 0)
                        duration_expvel.setText("")
                        duration_expvel.setError(resources.getText(R.string.error_validation_required))
                        duration_expvel.requestFocus()
                    } else {
                        exercisesTypes.remove(resources.getString(R.string.EXPVEL_KEY))
                        duration_expvel.setText("")
                        duration_expvel.setError(null)
                        duration_expvel.visibility = View.INVISIBLE
                    }
                }
                resources.getString(R.string.PROTR) -> {
                    if (isChecked) {
                        duration_protr.visibility = View.VISIBLE
                        exercisesTypes.put(resources.getString(R.string.PROTR_KEY), 0)
                        duration_protr.setText("")
                        duration_protr.setError(resources.getText(R.string.error_validation_required))
                        duration_protr.requestFocus()
                    } else {
                        exercisesTypes.remove(resources.getString(R.string.PROTR_KEY))
                        duration_protr.setText("")
                        duration_protr.setError(null)
                        duration_protr.visibility = View.INVISIBLE
                    }
                }
            }
        }
    }
}
