package com.jesuscb.footcoach.view.training

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.support.design.widget.FloatingActionButton
import android.support.v4.app.Fragment
import android.support.v4.widget.SwipeRefreshLayout
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import android.widget.ProgressBar
import android.widget.Toast
import com.jesuscb.footcoach.R
import com.jesuscb.footcoach.model.Training
import com.jesuscb.footcoach.presenter.training.TrainingListPresenter
import com.jesuscb.footcoach.view.core.Utils
import kotlinx.android.synthetic.main.fragment_list_trainings.*

class TrainingListFragment : Fragment(), TrainingListView {

    private lateinit var trainingAdapter: TrainingAdapter
    private lateinit var trainingListPresenter: TrainingListPresenter
    private lateinit var swipeRefreshLayout: SwipeRefreshLayout
    private lateinit var progressBar: ProgressBar

    private val TAG_ADD_TRAINING_RESULT = 1
    private val TAG_DETAILS_TRAINING_RESULT = 2

    private val TAG = "TrainingListFragment"

    @Override
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState);
        trainingListPresenter = TrainingListPresenter(this)
        trainingListPresenter.start()
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        // Inflate the layout for this fragment
        var view: View = inflater.inflate(R.layout.fragment_list_trainings, container, false)

        //RecyclerView
        val rv = view.findViewById<RecyclerView>(R.id.list_trainings)
        rv.layoutManager = LinearLayoutManager(context, LinearLayout.VERTICAL, false)
        rv.adapter = trainingAdapter

        //Floating Action Button
        val fab: FloatingActionButton = view.findViewById<FloatingActionButton>(R.id.fab_add_training)
        fab.setOnClickListener { newTraining() }

        //Swipe Refresh Layout
        swipeRefreshLayout = view.findViewById<SwipeRefreshLayout>(R.id.swipe_list_trainings);
        progressBar = view.findViewById<ProgressBar>(R.id.progressBar_trainings);
        initSwipeToRefreshView()

        progressBar.visibility = View.VISIBLE
        //Get training list
        trainingListPresenter.getTrainings()
        return view
    }

    override fun initialise() {
        trainingAdapter = TrainingAdapter(mutableListOf(), this)
    }

    private fun newTraining() {
        val intent = TrainingCRUDActivity.newIntent(context!!, "")
        startActivityForResult(intent, TAG_ADD_TRAINING_RESULT)
    }

    fun viewDetailsTraining(id: String) {
        val intent = TrainingDetailActivity.newIntent(context!!, id)
        startActivityForResult(intent, TAG_DETAILS_TRAINING_RESULT)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        when (requestCode) {
            TAG_ADD_TRAINING_RESULT -> {
                when (resultCode) {
                    Activity.RESULT_OK -> {
                        Toast.makeText(context, getString(R.string.success_adding_training), Toast.LENGTH_SHORT).show()
                        trainingListPresenter.getTrainings()
                    }
                    Activity.RESULT_CANCELED -> {
                        Log.i(TAG, "Operación cancelada")
                    }
                    else -> {
                        showToastErrorAddTraining()
                    }
                }
            }
            TAG_DETAILS_TRAINING_RESULT -> {
                when (resultCode) {
                    Activity.RESULT_OK -> {
                        Toast.makeText(context, getString(R.string.success_deleting_training), Toast.LENGTH_SHORT).show()
                        trainingListPresenter.getTrainings()
                    }
                    Activity.RESULT_CANCELED -> {
                        Log.i(TAG, "Operación cancelada")
                    }
                    else -> {
                       // showToastErrorAddTraining()
                    }
                }
            }
        }
    }

    private fun showToastErrorAddTraining() {
        Toast.makeText(context, getString(R.string.error_adding_training), Toast.LENGTH_SHORT).show()
    }

    override fun updateList(trainingList: MutableList<Training>) {
        val trainingListOrdered = trainingList.sortedWith(Utils.CompareTrainingsDatesDescending).toMutableList()
        this.trainingAdapter.setItems(trainingListOrdered)
        this.trainingAdapter.notifyDataSetChanged()
        if (trainingList.isEmpty()) {
            emptyState.visibility = View.VISIBLE
        } else {
            emptyState.visibility = View.GONE
        }
    }

    // Initialize my Swipe-to-refresh
    private fun initSwipeToRefreshView() {
        swipeRefreshLayout.setEnabled(true)
        swipeRefreshLayout.setOnRefreshListener(
                SwipeRefreshLayout.OnRefreshListener {
                    trainingListPresenter.getTrainings()
                }
        )
    }

    override fun showRefreshProgressBar() {
        if (!swipeRefreshLayout.isRefreshing()) {
            swipeRefreshLayout.post(Runnable {
                swipeRefreshLayout.setRefreshing(true)
                swipeRefreshLayout.setEnabled(false)
            })
        }
    }

    override fun hideRefreshProgressBar() {
        if (swipeRefreshLayout.isRefreshing()) {
            swipeRefreshLayout.post(Runnable {
                swipeRefreshLayout.setEnabled(true)
                swipeRefreshLayout.setRefreshing(false)
            })
        }
        progressBar.visibility = View.GONE
    }

}